"""Suite Standard Suite: Common terms for most applications
Level 1, version 1

Generated from /Volumes/Sap/Applications (Mac OS 9)/Metrowerks CodeWarrior 7.0/Metrowerks CodeWarrior/CodeWarrior IDE 4.2.5
AETE/AEUT resource version 1/0, language 0, script 0
"""

import aetools
import MacOS

_code = 'CoRe'

from StdSuites.Standard_Suite import *
class Standard_Suite_Events(Standard_Suite_Events):

    _argmap_close = {
        'saving' : 'savo',
        'saving_in' : 'kfil',
    }

    def close(self, _object, _attributes={}, **_arguments):
        """close: close an object
        Required argument: the object to close
        Keyword argument saving: specifies whether or not changes should be saved before closing
        Keyword argument saving_in: the file in which to save the object
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'core'
        _subcode = 'clos'

        aetools.keysubst(_arguments, self._argmap_close)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'savo', _Enum_savo)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_count = {
        'each' : 'kocl',
    }

    def count(self, _object, _attributes={}, **_arguments):
        """count: return the number of elements of a particular class within an object
        Required argument: the object whose elements are to be counted
        Keyword argument each: the class of the elements to be counted. Keyword 'each' is optional in AppleScript
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: the number of elements
        """
        _code = 'core'
        _subcode = 'cnte'

        aetools.keysubst(_arguments, self._argmap_count)
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_get = {
        'as' : 'rtyp',
    }

    def get(self, _object, _attributes={}, **_arguments):
        """get: get the data for an object
        Required argument: the object whose data is to be returned
        Keyword argument as: the desired types for the data, in order of preference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: The data from the object
        """
        _code = 'core'
        _subcode = 'getd'

        aetools.keysubst(_arguments, self._argmap_get)
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_make = {
        'new' : 'kocl',
        'as' : 'rtyp',
        'at' : 'insh',
        'with_data' : 'data',
        'with_properties' : 'prdt',
    }

    def make(self, _no_object=None, _attributes={}, **_arguments):
        """make: make a new element
        Keyword argument new: the class of the new element\xd1keyword 'new' is optional in AppleScript
        Keyword argument as: the desired types for the data, in order of preference
        Keyword argument at: the location at which to insert the element
        Keyword argument with_data: the initial data for the element
        Keyword argument with_properties: the initial values for the properties of the element
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: to the new object(s)
        """
        _code = 'core'
        _subcode = 'crel'

        aetools.keysubst(_arguments, self._argmap_make)
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def select(self, _object=None, _attributes={}, **_arguments):
        """select: select the specified object
        Required argument: the object to select
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'misc'
        _subcode = 'slct'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_set = {
        'to' : 'data',
    }

    def set(self, _object, _attributes={}, **_arguments):
        """set: set an object's data
        Required argument: the object to change
        Keyword argument to: the new value
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'core'
        _subcode = 'setd'

        aetools.keysubst(_arguments, self._argmap_set)
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']


class application(aetools.ComponentItem):
    """application - an application program """
    want = 'capp'
class _Prop_user_interaction(aetools.NProperty):
    """user interaction - user interaction level """
    which = 'inte'
    want = 'Inte'
user_interaction = _Prop_user_interaction()
#        element 'cwin' as ['indx', 'name', 'rang']
#        element 'docu' as ['indx', 'name', 'rang']

class character(aetools.ComponentItem):
    """character - a character """
    want = 'cha '
class _Prop_length(aetools.NProperty):
    """length - length in characters of this object """
    which = 'pLen'
    want = 'long'
class _Prop_offset(aetools.NProperty):
    """offset - offset of a text object from the beginning of the document (first char has offset 1) """
    which = 'pOff'
    want = 'long'

class insertion_point(aetools.ComponentItem):
    """insertion point - An insertion location between two objects """
    want = 'cins'

class line(aetools.ComponentItem):
    """line - lines of text """
    want = 'clin'
class _Prop_index(aetools.NProperty):
    """index - index of a line object from the beginning of the document (first line has index 1) """
    which = 'pidx'
    want = 'long'
#        element 'cha ' as ['indx', 'rang', 'rele']

lines = line

class selection_2d_object(aetools.ComponentItem):
    """selection-object - the selection visible to the user """
    want = 'csel'
class _Prop_contents(aetools.NProperty):
    """contents - the contents of the selection """
    which = 'pcnt'
    want = 'type'
#        element 'cha ' as ['indx', 'rele', 'rang', 'test']
#        element 'clin' as ['indx', 'rang', 'rele']
#        element 'ctxt' as ['rang']

class text(aetools.ComponentItem):
    """text - Text """
    want = 'ctxt'
#        element 'cha ' as ['indx', 'rele', 'rang']
#        element 'cins' as ['rele']
#        element 'clin' as ['indx', 'rang', 'rele']
#        element 'ctxt' as ['rang']

class window(aetools.ComponentItem):
    """window - A window """
    want = 'cwin'
class _Prop_bounds(aetools.NProperty):
    """bounds - the boundary rectangle for the window """
    which = 'pbnd'
    want = 'qdrt'
class _Prop_document(aetools.NProperty):
    """document - the document that owns this window """
    which = 'docu'
    want = 'docu'
class _Prop_name(aetools.NProperty):
    """name - the title of the window """
    which = 'pnam'
    want = 'itxt'
class _Prop_position(aetools.NProperty):
    """position - upper left coordinates of window """
    which = 'ppos'
    want = 'QDpt'
class _Prop_visible(aetools.NProperty):
    """visible - is the window visible? """
    which = 'pvis'
    want = 'bool'
class _Prop_zoomed(aetools.NProperty):
    """zoomed - Is the window zoomed? """
    which = 'pzum'
    want = 'bool'

windows = window

class document(aetools.ComponentItem):
    """document - a document """
    want = 'docu'
class _Prop_file_permissions(aetools.NProperty):
    """file permissions - the file permissions for the document """
    which = 'PERM'
    want = 'PERM'
class _Prop_kind(aetools.NProperty):
    """kind - the kind of document """
    which = 'DKND'
    want = 'DKND'
class _Prop_location(aetools.NProperty):
    """location - the file of the document """
    which = 'FILE'
    want = 'fss '
class _Prop_window(aetools.NProperty):
    """window - the window of the document. """
    which = 'cwin'
    want = 'cwin'

documents = document

class files(aetools.ComponentItem):
    """files - Every file """
    want = 'file'

file = files
application._superclassnames = []
application._privpropdict = {
    'user_interaction' : _Prop_user_interaction,
}
application._privelemdict = {
    'document' : document,
    'window' : window,
}
character._superclassnames = []
character._privpropdict = {
    'length' : _Prop_length,
    'offset' : _Prop_offset,
}
character._privelemdict = {
}
insertion_point._superclassnames = []
insertion_point._privpropdict = {
    'length' : _Prop_length,
    'offset' : _Prop_offset,
}
insertion_point._privelemdict = {
}
line._superclassnames = []
line._privpropdict = {
    'index' : _Prop_index,
    'length' : _Prop_length,
    'offset' : _Prop_offset,
}
line._privelemdict = {
    'character' : character,
}
selection_2d_object._superclassnames = []
selection_2d_object._privpropdict = {
    'contents' : _Prop_contents,
    'length' : _Prop_length,
    'offset' : _Prop_offset,
}
selection_2d_object._privelemdict = {
    'character' : character,
    'line' : line,
    'text' : text,
}
text._superclassnames = []
text._privpropdict = {
    'length' : _Prop_length,
    'offset' : _Prop_offset,
}
text._privelemdict = {
    'character' : character,
    'insertion_point' : insertion_point,
    'line' : line,
    'text' : text,
}
window._superclassnames = []
window._privpropdict = {
    'bounds' : _Prop_bounds,
    'document' : _Prop_document,
    'index' : _Prop_index,
    'name' : _Prop_name,
    'position' : _Prop_position,
    'visible' : _Prop_visible,
    'zoomed' : _Prop_zoomed,
}
window._privelemdict = {
}
document._superclassnames = []
document._privpropdict = {
    'file_permissions' : _Prop_file_permissions,
    'index' : _Prop_index,
    'kind' : _Prop_kind,
    'location' : _Prop_location,
    'name' : _Prop_name,
    'window' : _Prop_window,
}
document._privelemdict = {
}
files._superclassnames = []
files._privpropdict = {
}
files._privelemdict = {
}

#
# Indices of types declared in this module
#
_classdeclarations = {
    'capp' : application,
    'cha ' : character,
    'cins' : insertion_point,
    'clin' : line,
    'csel' : selection_2d_object,
    'ctxt' : text,
    'cwin' : window,
    'docu' : document,
    'file' : files,
}

_propdeclarations = {
    'DKND' : _Prop_kind,
    'FILE' : _Prop_location,
    'PERM' : _Prop_file_permissions,
    'cwin' : _Prop_window,
    'docu' : _Prop_document,
    'inte' : _Prop_user_interaction,
    'pLen' : _Prop_length,
    'pOff' : _Prop_offset,
    'pbnd' : _Prop_bounds,
    'pcnt' : _Prop_contents,
    'pidx' : _Prop_index,
    'pnam' : _Prop_name,
    'ppos' : _Prop_position,
    'pvis' : _Prop_visible,
    'pzum' : _Prop_zoomed,
}

_compdeclarations = {
}

_enumdeclarations = {
}



"""Test suite for the profile module."""

import sys
import pstats
import unittest
from StringIO import StringIO
from test.test_support import run_unittest

import profile
from test.profilee import testfunc, timer


class ProfileTest(unittest.TestCase):

    profilerclass = profile.Profile
    methodnames = ['print_stats', 'print_callers', 'print_callees']
    expected_output = {}
    expected_list_sort_output = ':0(sort)'

    @classmethod
    def do_profiling(cls):
        results = []
        prof = cls.profilerclass(timer, 0.001)
        start_timer = timer()
        prof.runctx("testfunc()", globals(), locals())
        results.append(timer() - start_timer)
        for methodname in cls.methodnames:
            s = StringIO()
            stats = pstats.Stats(prof, stream=s)
            stats.strip_dirs().sort_stats("stdname")
            getattr(stats, methodname)()
            results.append(s.getvalue())
        return results

    def test_cprofile(self):
        results = self.do_profiling()
        self.assertEqual(results[0], 1000)
        for i, method in enumerate(self.methodnames):
            self.assertEqual(results[i+1], self.expected_output[method],
                             "Stats.%s output for %s doesn't fit expectation!" %
                             (method, self.profilerclass.__name__))

    def test_calling_conventions(self):
        # Issue #5330: profile and cProfile wouldn't report C functions called
        # with keyword arguments. We test all calling conventions.
        stmts = [
            "[].sort()",
            "[].sort(reverse=True)",
            "[].sort(*(None, None, True))",
            "[].sort(**dict(reverse=True))",
        ]
        for stmt in stmts:
            s = StringIO()
            prof = self.profilerclass(timer, 0.001)
            prof.runctx(stmt, globals(), locals())
            stats = pstats.Stats(prof, stream=s)
            stats.print_stats()
            res = s.getvalue()
            self.assertIn(self.expected_list_sort_output, res,
                "Profiling {0!r} didn't report list.sort:\n{1}".format(stmt, res))


def regenerate_expected_output(filename, cls):
    filename = filename.rstrip('co')
    print 'Regenerating %s...' % filename
    results = cls.do_profiling()

    newfile = []
    with open(filename, 'r') as f:
        for line in f:
            newfile.append(line)
            if line[:6] == '#--cut':
                break

    with open(filename, 'w') as f:
        f.writelines(newfile)
        for i, method in enumerate(cls.methodnames):
            f.write('%s.expected_output[%r] = """\\\n%s"""\n' % (
                cls.__name__, method, results[i+1]))
        f.write('\nif __name__ == "__main__":\n    main()\n')


def test_main():
    run_unittest(ProfileTest)

def main():
    if '-r' not in sys.argv:
        test_main()
    else:
        regenerate_expected_output(__file__, ProfileTest)


# Don't remove this comment. Everything below it is auto-generated.
#--cut--------------------------------------------------------------------------
ProfileTest.expected_output['print_stats'] = """\
         127 function calls (107 primitive calls) in 999.749 seconds

   Ordered by: standard name

   ncalls  tottime  percall  cumtime  percall filename:lineno(function)
        4   -0.004   -0.001   -0.004   -0.001 :0(append)
        4   -0.004   -0.001   -0.004   -0.001 :0(exc_info)
       12   -0.024   -0.002   11.964    0.997 :0(hasattr)
        8   -0.008   -0.001   -0.008   -0.001 :0(range)
        1    0.000    0.000    0.000    0.000 :0(setprofile)
        1   -0.002   -0.002  999.751  999.751 <string>:1(<module>)
        0    0.000             0.000          profile:0(profiler)
        1   -0.002   -0.002  999.749  999.749 profile:0(testfunc())
       28   27.972    0.999   27.972    0.999 profilee.py:110(__getattr__)
        1  269.996  269.996  999.753  999.753 profilee.py:25(testfunc)
     23/3  149.937    6.519  169.917   56.639 profilee.py:35(factorial)
       20   19.980    0.999   19.980    0.999 profilee.py:48(mul)
        2   39.986   19.993  599.814  299.907 profilee.py:55(helper)
        4  115.984   28.996  119.964   29.991 profilee.py:73(helper1)
        2   -0.006   -0.003  139.942   69.971 profilee.py:84(helper2_indirect)
        8  311.976   38.997  399.896   49.987 profilee.py:88(helper2)
        8   63.968    7.996   79.944    9.993 profilee.py:98(subhelper)


"""
ProfileTest.expected_output['print_callers'] = """\
   Ordered by: standard name

Function                          was called by...
:0(append)                        <- profilee.py:73(helper1)(4)  119.964
:0(exc_info)                      <- profilee.py:73(helper1)(4)  119.964
:0(hasattr)                       <- profilee.py:73(helper1)(4)  119.964
                                     profilee.py:88(helper2)(8)  399.896
:0(range)                         <- profilee.py:98(subhelper)(8)   79.944
:0(setprofile)                    <- profile:0(testfunc())(1)  999.749
<string>:1(<module>)              <- profile:0(testfunc())(1)  999.749
profile:0(profiler)               <-
profile:0(testfunc())             <- profile:0(profiler)(1)    0.000
profilee.py:110(__getattr__)      <- :0(hasattr)(12)   11.964
                                     profilee.py:98(subhelper)(16)   79.944
profilee.py:25(testfunc)          <- <string>:1(<module>)(1)  999.751
profilee.py:35(factorial)         <- profilee.py:25(testfunc)(1)  999.753
                                     profilee.py:35(factorial)(20)  169.917
                                     profilee.py:84(helper2_indirect)(2)  139.942
profilee.py:48(mul)               <- profilee.py:35(factorial)(20)  169.917
profilee.py:55(helper)            <- profilee.py:25(testfunc)(2)  999.753
profilee.py:73(helper1)           <- profilee.py:55(helper)(4)  599.814
profilee.py:84(helper2_indirect)  <- profilee.py:55(helper)(2)  599.814
profilee.py:88(helper2)           <- profilee.py:55(helper)(6)  599.814
                                     profilee.py:84(helper2_indirect)(2)  139.942
profilee.py:98(subhelper)         <- profilee.py:88(helper2)(8)  399.896


"""
ProfileTest.expected_output['print_callees'] = """\
   Ordered by: standard name

Function                          called...
:0(append)                        ->
:0(exc_info)                      ->
:0(hasattr)                       -> profilee.py:110(__getattr__)(12)   27.972
:0(range)                         ->
:0(setprofile)                    ->
<string>:1(<module>)              -> profilee.py:25(testfunc)(1)  999.753
profile:0(profiler)               -> profile:0(testfunc())(1)  999.749
profile:0(testfunc())             -> :0(setprofile)(1)    0.000
                                     <string>:1(<module>)(1)  999.751
profilee.py:110(__getattr__)      ->
profilee.py:25(testfunc)          -> profilee.py:35(factorial)(1)  169.917
                                     profilee.py:55(helper)(2)  599.814
profilee.py:35(factorial)         -> profilee.py:35(factorial)(20)  169.917
                                     profilee.py:48(mul)(20)   19.980
profilee.py:48(mul)               ->
profilee.py:55(helper)            -> profilee.py:73(helper1)(4)  119.964
                                     profilee.py:84(helper2_indirect)(2)  139.942
                                     profilee.py:88(helper2)(6)  399.896
profilee.py:73(helper1)           -> :0(append)(4)   -0.004
                                     :0(exc_info)(4)   -0.004
                                     :0(hasattr)(4)   11.964
profilee.py:84(helper2_indirect)  -> profilee.py:35(factorial)(2)  169.917
                                     profilee.py:88(helper2)(2)  399.896
profilee.py:88(helper2)           -> :0(hasattr)(8)   11.964
                                     profilee.py:98(subhelper)(8)   79.944
profilee.py:98(subhelper)         -> :0(range)(8)   -0.008
                                     profilee.py:110(__getattr__)(16)   27.972


"""

if __name__ == "__main__":
    main()



"""Tests for distutils.command.build_scripts."""

import os
import unittest

from distutils.command.build_scripts import build_scripts
from distutils.core import Distribution
import sysconfig

from distutils.tests import support
from test.test_support import run_unittest


class BuildScriptsTestCase(support.TempdirManager,
                           support.LoggingSilencer,
                           unittest.TestCase):

    def test_default_settings(self):
        cmd = self.get_build_scripts_cmd("/foo/bar", [])
        self.assertFalse(cmd.force)
        self.assertIsNone(cmd.build_dir)

        cmd.finalize_options()

        self.assertTrue(cmd.force)
        self.assertEqual(cmd.build_dir, "/foo/bar")

    def test_build(self):
        source = self.mkdtemp()
        target = self.mkdtemp()
        expected = self.write_sample_scripts(source)

        cmd = self.get_build_scripts_cmd(target,
                                         [os.path.join(source, fn)
                                          for fn in expected])
        cmd.finalize_options()
        cmd.run()

        built = os.listdir(target)
        for name in expected:
            self.assertIn(name, built)

    def get_build_scripts_cmd(self, target, scripts):
        import sys
        dist = Distribution()
        dist.scripts = scripts
        dist.command_obj["build"] = support.DummyCommand(
            build_scripts=target,
            force=1,
            executable=sys.executable
            )
        return build_scripts(dist)

    def write_sample_scripts(self, dir):
        expected = []
        expected.append("script1.py")
        self.write_script(dir, "script1.py",
                          ("#! /usr/bin/env python2.3\n"
                           "# bogus script w/ Python sh-bang\n"
                           "pass\n"))
        expected.append("script2.py")
        self.write_script(dir, "script2.py",
                          ("#!/usr/bin/python\n"
                           "# bogus script w/ Python sh-bang\n"
                           "pass\n"))
        expected.append("shell.sh")
        self.write_script(dir, "shell.sh",
                          ("#!/bin/sh\n"
                           "# bogus shell script w/ sh-bang\n"
                           "exit 0\n"))
        return expected

    def write_script(self, dir, name, text):
        f = open(os.path.join(dir, name), "w")
        try:
            f.write(text)
        finally:
            f.close()

    def test_version_int(self):
        source = self.mkdtemp()
        target = self.mkdtemp()
        expected = self.write_sample_scripts(source)


        cmd = self.get_build_scripts_cmd(target,
                                         [os.path.join(source, fn)
                                          for fn in expected])
        cmd.finalize_options()

        # http://bugs.python.org/issue4524
        #
        # On linux-g++-32 with command line `./configure --enable-ipv6
        # --with-suffix=3`, python is compiled okay but the build scripts
        # failed when writing the name of the executable
        old = sysconfig.get_config_vars().get('VERSION')
        sysconfig._CONFIG_VARS['VERSION'] = 4
        try:
            cmd.run()
        finally:
            if old is not None:
                sysconfig._CONFIG_VARS['VERSION'] = old

        built = os.listdir(target)
        for name in expected:
            self.assertIn(name, built)

def test_suite():
    return unittest.makeSuite(BuildScriptsTestCase)

if __name__ == "__main__":
    run_unittest(test_suite())



from test import test_support
import sys
import unicodedata
import unittest
import urlparse

RFC1808_BASE = "http://a/b/c/d;p?q#f"
RFC2396_BASE = "http://a/b/c/d;p?q"
RFC3986_BASE = 'http://a/b/c/d;p?q'
SIMPLE_BASE  = 'http://a/b/c/d'

# A list of test cases.  Each test case is a two-tuple that contains
# a string with the query and a dictionary with the expected result.

parse_qsl_test_cases = [
    ("", []),
    ("&", []),
    ("&&", []),
    ("=", [('', '')]),
    ("=a", [('', 'a')]),
    ("a", [('a', '')]),
    ("a=", [('a', '')]),
    ("a=", [('a', '')]),
    ("&a=b", [('a', 'b')]),
    ("a=a+b&b=b+c", [('a', 'a b'), ('b', 'b c')]),
    ("a=1&a=2", [('a', '1'), ('a', '2')]),
    (b"", []),
    (b"&", []),
    (b"&&", []),
    (b"=", [(b'', b'')]),
    (b"=a", [(b'', b'a')]),
    (b"a", [(b'a', b'')]),
    (b"a=", [(b'a', b'')]),
    (b"&a=b", [(b'a', b'b')]),
    (b"a=a+b&b=b+c", [(b'a', b'a b'), (b'b', b'b c')]),
    (b"a=1&a=2", [(b'a', b'1'), (b'a', b'2')]),
    (";a=b", [(';a', 'b')]),
    ("a=a+b;b=b+c", [('a', 'a b;b=b c')]),
    (b";a=b", [(b';a', b'b')]),
    (b"a=a+b;b=b+c", [(b'a', b'a b;b=b c')]),
]

parse_qs_test_cases = [
    ("", {}),
    ("&", {}),
    ("&&", {}),
    ("=", {'': ['']}),
    ("=a", {'': ['a']}),
    ("a", {'a': ['']}),
    ("a=", {'a': ['']}),
    ("&a=b", {'a': ['b']}),
    ("a=a+b&b=b+c", {'a': ['a b'], 'b': ['b c']}),
    ("a=1&a=2", {'a': ['1', '2']}),
    (b"", {}),
    (b"&", {}),
    (b"&&", {}),
    (b"=", {b'': [b'']}),
    (b"=a", {b'': [b'a']}),
    (b"a", {b'a': [b'']}),
    (b"a=", {b'a': [b'']}),
    (b"&a=b", {b'a': [b'b']}),
    (b"a=a+b&b=b+c", {b'a': [b'a b'], b'b': [b'b c']}),
    (b"a=1&a=2", {b'a': [b'1', b'2']}),
    (";a=b", {';a': ['b']}),
    ("a=a+b;b=b+c", {'a': ['a b;b=b c']}),
    (b";a=b", {b';a': [b'b']}),
    (b"a=a+b;b=b+c", {b'a':[ b'a b;b=b c']}),
]

class UrlParseTestCase(unittest.TestCase):

    def checkRoundtrips(self, url, parsed, split):
        result = urlparse.urlparse(url)
        self.assertEqual(result, parsed)
        t = (result.scheme, result.netloc, result.path,
             result.params, result.query, result.fragment)
        self.assertEqual(t, parsed)
        # put it back together and it should be the same
        result2 = urlparse.urlunparse(result)
        self.assertEqual(result2, url)
        self.assertEqual(result2, result.geturl())

        # the result of geturl() is a fixpoint; we can always parse it
        # again to get the same result:
        result3 = urlparse.urlparse(result.geturl())
        self.assertEqual(result3.geturl(), result.geturl())
        self.assertEqual(result3,          result)
        self.assertEqual(result3.scheme,   result.scheme)
        self.assertEqual(result3.netloc,   result.netloc)
        self.assertEqual(result3.path,     result.path)
        self.assertEqual(result3.params,   result.params)
        self.assertEqual(result3.query,    result.query)
        self.assertEqual(result3.fragment, result.fragment)
        self.assertEqual(result3.username, result.username)
        self.assertEqual(result3.password, result.password)
        self.assertEqual(result3.hostname, result.hostname)
        self.assertEqual(result3.port,     result.port)

        # check the roundtrip using urlsplit() as well
        result = urlparse.urlsplit(url)
        self.assertEqual(result, split)
        t = (result.scheme, result.netloc, result.path,
             result.query, result.fragment)
        self.assertEqual(t, split)
        result2 = urlparse.urlunsplit(result)
        self.assertEqual(result2, url)
        self.assertEqual(result2, result.geturl())

        # check the fixpoint property of re-parsing the result of geturl()
        result3 = urlparse.urlsplit(result.geturl())
        self.assertEqual(result3.geturl(), result.geturl())
        self.assertEqual(result3,          result)
        self.assertEqual(result3.scheme,   result.scheme)
        self.assertEqual(result3.netloc,   result.netloc)
        self.assertEqual(result3.path,     result.path)
        self.assertEqual(result3.query,    result.query)
        self.assertEqual(result3.fragment, result.fragment)
        self.assertEqual(result3.username, result.username)
        self.assertEqual(result3.password, result.password)
        self.assertEqual(result3.hostname, result.hostname)
        self.assertEqual(result3.port,     result.port)

    def test_qsl(self):
        for orig, expect in parse_qsl_test_cases:
            result = urlparse.parse_qsl(orig, keep_blank_values=True)
            self.assertEqual(result, expect, "Error parsing %r" % orig)
            expect_without_blanks = [v for v in expect if len(v[1])]
            result = urlparse.parse_qsl(orig, keep_blank_values=False)
            self.assertEqual(result, expect_without_blanks,
                    "Error parsing %r" % orig)

    def test_qs(self):
        for orig, expect in parse_qs_test_cases:
            result = urlparse.parse_qs(orig, keep_blank_values=True)
            self.assertEqual(result, expect, "Error parsing %r" % orig)
            expect_without_blanks = dict(
                    [(v, expect[v]) for v in expect if len(expect[v][0])])
            result = urlparse.parse_qs(orig, keep_blank_values=False)
            self.assertEqual(result, expect_without_blanks,
                    "Error parsing %r" % orig)

    def test_roundtrips(self):
        testcases = [
            ('file:///tmp/junk.txt',
             ('file', '', '/tmp/junk.txt', '', '', ''),
             ('file', '', '/tmp/junk.txt', '', '')),
            ('imap://mail.python.org/mbox1',
             ('imap', 'mail.python.org', '/mbox1', '', '', ''),
             ('imap', 'mail.python.org', '/mbox1', '', '')),
            ('mms://wms.sys.hinet.net/cts/Drama/09006251100.asf',
             ('mms', 'wms.sys.hinet.net', '/cts/Drama/09006251100.asf',
              '', '', ''),
             ('mms', 'wms.sys.hinet.net', '/cts/Drama/09006251100.asf',
              '', '')),
            ('nfs://server/path/to/file.txt',
             ('nfs', 'server', '/path/to/file.txt',  '', '', ''),
             ('nfs', 'server', '/path/to/file.txt', '', '')),
            ('svn+ssh://svn.zope.org/repos/main/ZConfig/trunk/',
             ('svn+ssh', 'svn.zope.org', '/repos/main/ZConfig/trunk/',
              '', '', ''),
             ('svn+ssh', 'svn.zope.org', '/repos/main/ZConfig/trunk/',
              '', '')),
            ('git+ssh://git@github.com/user/project.git',
            ('git+ssh', 'git@github.com','/user/project.git',
             '','',''),
            ('git+ssh', 'git@github.com','/user/project.git',
             '', ''))
            ]
        for url, parsed, split in testcases:
            self.checkRoundtrips(url, parsed, split)

    def test_http_roundtrips(self):
        # urlparse.urlsplit treats 'http:' as an optimized special case,
        # so we test both 'http:' and 'https:' in all the following.
        # Three cheers for white box knowledge!
        testcases = [
            ('://www.python.org',
             ('www.python.org', '', '', '', ''),
             ('www.python.org', '', '', '')),
            ('://www.python.org#abc',
             ('www.python.org', '', '', '', 'abc'),
             ('www.python.org', '', '', 'abc')),
            ('://www.python.org?q=abc',
             ('www.python.org', '', '', 'q=abc', ''),
             ('www.python.org', '', 'q=abc', '')),
            ('://www.python.org/#abc',
             ('www.python.org', '/', '', '', 'abc'),
             ('www.python.org', '/', '', 'abc')),
            ('://a/b/c/d;p?q#f',
             ('a', '/b/c/d', 'p', 'q', 'f'),
             ('a', '/b/c/d;p', 'q', 'f')),
            ]
        for scheme in ('http', 'https'):
            for url, parsed, split in testcases:
                url = scheme + url
                parsed = (scheme,) + parsed
                split = (scheme,) + split
                self.checkRoundtrips(url, parsed, split)

    def checkJoin(self, base, relurl, expected):
        self.assertEqual(urlparse.urljoin(base, relurl), expected,
                         (base, relurl, expected))

    def test_unparse_parse(self):
        for u in ['Python', './Python','x-newscheme://foo.com/stuff','x://y','x:/y','x:/','/',]:
            self.assertEqual(urlparse.urlunsplit(urlparse.urlsplit(u)), u)
            self.assertEqual(urlparse.urlunparse(urlparse.urlparse(u)), u)

    def test_RFC1808(self):
        # "normal" cases from RFC 1808:
        self.checkJoin(RFC1808_BASE, 'g:h', 'g:h')
        self.checkJoin(RFC1808_BASE, 'g', 'http://a/b/c/g')
        self.checkJoin(RFC1808_BASE, './g', 'http://a/b/c/g')
        self.checkJoin(RFC1808_BASE, 'g/', 'http://a/b/c/g/')
        self.checkJoin(RFC1808_BASE, '/g', 'http://a/g')
        self.checkJoin(RFC1808_BASE, '//g', 'http://g')
        self.checkJoin(RFC1808_BASE, 'g?y', 'http://a/b/c/g?y')
        self.checkJoin(RFC1808_BASE, 'g?y/./x', 'http://a/b/c/g?y/./x')
        self.checkJoin(RFC1808_BASE, '#s', 'http://a/b/c/d;p?q#s')
        self.checkJoin(RFC1808_BASE, 'g#s', 'http://a/b/c/g#s')
        self.checkJoin(RFC1808_BASE, 'g#s/./x', 'http://a/b/c/g#s/./x')
        self.checkJoin(RFC1808_BASE, 'g?y#s', 'http://a/b/c/g?y#s')
        self.checkJoin(RFC1808_BASE, 'g;x', 'http://a/b/c/g;x')
        self.checkJoin(RFC1808_BASE, 'g;x?y#s', 'http://a/b/c/g;x?y#s')
        self.checkJoin(RFC1808_BASE, '.', 'http://a/b/c/')
        self.checkJoin(RFC1808_BASE, './', 'http://a/b/c/')
        self.checkJoin(RFC1808_BASE, '..', 'http://a/b/')
        self.checkJoin(RFC1808_BASE, '../', 'http://a/b/')
        self.checkJoin(RFC1808_BASE, '../g', 'http://a/b/g')
        self.checkJoin(RFC1808_BASE, '../..', 'http://a/')
        self.checkJoin(RFC1808_BASE, '../../', 'http://a/')
        self.checkJoin(RFC1808_BASE, '../../g', 'http://a/g')

        # "abnormal" cases from RFC 1808:
        self.checkJoin(RFC1808_BASE, '', 'http://a/b/c/d;p?q#f')
        self.checkJoin(RFC1808_BASE, '../../../g', 'http://a/../g')
        self.checkJoin(RFC1808_BASE, '../../../../g', 'http://a/../../g')
        self.checkJoin(RFC1808_BASE, '/./g', 'http://a/./g')
        self.checkJoin(RFC1808_BASE, '/../g', 'http://a/../g')
        self.checkJoin(RFC1808_BASE, 'g.', 'http://a/b/c/g.')
        self.checkJoin(RFC1808_BASE, '.g', 'http://a/b/c/.g')
        self.checkJoin(RFC1808_BASE, 'g..', 'http://a/b/c/g..')
        self.checkJoin(RFC1808_BASE, '..g', 'http://a/b/c/..g')
        self.checkJoin(RFC1808_BASE, './../g', 'http://a/b/g')
        self.checkJoin(RFC1808_BASE, './g/.', 'http://a/b/c/g/')
        self.checkJoin(RFC1808_BASE, 'g/./h', 'http://a/b/c/g/h')
        self.checkJoin(RFC1808_BASE, 'g/../h', 'http://a/b/c/h')

        # RFC 1808 and RFC 1630 disagree on these (according to RFC 1808),
        # so we'll not actually run these tests (which expect 1808 behavior).
        #self.checkJoin(RFC1808_BASE, 'http:g', 'http:g')
        #self.checkJoin(RFC1808_BASE, 'http:', 'http:')

    def test_RFC2368(self):
        # Issue 11467: path that starts with a number is not parsed correctly
        self.assertEqual(urlparse.urlparse('mailto:1337@example.org'),
                ('mailto', '', '1337@example.org', '', '', ''))

    def test_RFC2396(self):
        # cases from RFC 2396
        self.checkJoin(RFC2396_BASE, 'g:h', 'g:h')
        self.checkJoin(RFC2396_BASE, 'g', 'http://a/b/c/g')
        self.checkJoin(RFC2396_BASE, './g', 'http://a/b/c/g')
        self.checkJoin(RFC2396_BASE, 'g/', 'http://a/b/c/g/')
        self.checkJoin(RFC2396_BASE, '/g', 'http://a/g')
        self.checkJoin(RFC2396_BASE, '//g', 'http://g')
        self.checkJoin(RFC2396_BASE, 'g?y', 'http://a/b/c/g?y')
        self.checkJoin(RFC2396_BASE, '#s', 'http://a/b/c/d;p?q#s')
        self.checkJoin(RFC2396_BASE, 'g#s', 'http://a/b/c/g#s')
        self.checkJoin(RFC2396_BASE, 'g?y#s', 'http://a/b/c/g?y#s')
        self.checkJoin(RFC2396_BASE, 'g;x', 'http://a/b/c/g;x')
        self.checkJoin(RFC2396_BASE, 'g;x?y#s', 'http://a/b/c/g;x?y#s')
        self.checkJoin(RFC2396_BASE, '.', 'http://a/b/c/')
        self.checkJoin(RFC2396_BASE, './', 'http://a/b/c/')
        self.checkJoin(RFC2396_BASE, '..', 'http://a/b/')
        self.checkJoin(RFC2396_BASE, '../', 'http://a/b/')
        self.checkJoin(RFC2396_BASE, '../g', 'http://a/b/g')
        self.checkJoin(RFC2396_BASE, '../..', 'http://a/')
        self.checkJoin(RFC2396_BASE, '../../', 'http://a/')
        self.checkJoin(RFC2396_BASE, '../../g', 'http://a/g')
        self.checkJoin(RFC2396_BASE, '', RFC2396_BASE)
        self.checkJoin(RFC2396_BASE, '../../../g', 'http://a/../g')
        self.checkJoin(RFC2396_BASE, '../../../../g', 'http://a/../../g')
        self.checkJoin(RFC2396_BASE, '/./g', 'http://a/./g')
        self.checkJoin(RFC2396_BASE, '/../g', 'http://a/../g')
        self.checkJoin(RFC2396_BASE, 'g.', 'http://a/b/c/g.')
        self.checkJoin(RFC2396_BASE, '.g', 'http://a/b/c/.g')
        self.checkJoin(RFC2396_BASE, 'g..', 'http://a/b/c/g..')
        self.checkJoin(RFC2396_BASE, '..g', 'http://a/b/c/..g')
        self.checkJoin(RFC2396_BASE, './../g', 'http://a/b/g')
        self.checkJoin(RFC2396_BASE, './g/.', 'http://a/b/c/g/')
        self.checkJoin(RFC2396_BASE, 'g/./h', 'http://a/b/c/g/h')
        self.checkJoin(RFC2396_BASE, 'g/../h', 'http://a/b/c/h')
        self.checkJoin(RFC2396_BASE, 'g;x=1/./y', 'http://a/b/c/g;x=1/y')
        self.checkJoin(RFC2396_BASE, 'g;x=1/../y', 'http://a/b/c/y')
        self.checkJoin(RFC2396_BASE, 'g?y/./x', 'http://a/b/c/g?y/./x')
        self.checkJoin(RFC2396_BASE, 'g?y/../x', 'http://a/b/c/g?y/../x')
        self.checkJoin(RFC2396_BASE, 'g#s/./x', 'http://a/b/c/g#s/./x')
        self.checkJoin(RFC2396_BASE, 'g#s/../x', 'http://a/b/c/g#s/../x')

    def test_RFC3986(self):
        # Test cases from RFC3986
        self.checkJoin(RFC3986_BASE, '?y','http://a/b/c/d;p?y')
        self.checkJoin(RFC2396_BASE, ';x', 'http://a/b/c/;x')
        self.checkJoin(RFC3986_BASE, 'g:h','g:h')
        self.checkJoin(RFC3986_BASE, 'g','http://a/b/c/g')
        self.checkJoin(RFC3986_BASE, './g','http://a/b/c/g')
        self.checkJoin(RFC3986_BASE, 'g/','http://a/b/c/g/')
        self.checkJoin(RFC3986_BASE, '/g','http://a/g')
        self.checkJoin(RFC3986_BASE, '//g','http://g')
        self.checkJoin(RFC3986_BASE, '?y','http://a/b/c/d;p?y')
        self.checkJoin(RFC3986_BASE, 'g?y','http://a/b/c/g?y')
        self.checkJoin(RFC3986_BASE, '#s','http://a/b/c/d;p?q#s')
        self.checkJoin(RFC3986_BASE, 'g#s','http://a/b/c/g#s')
        self.checkJoin(RFC3986_BASE, 'g?y#s','http://a/b/c/g?y#s')
        self.checkJoin(RFC3986_BASE, ';x','http://a/b/c/;x')
        self.checkJoin(RFC3986_BASE, 'g;x','http://a/b/c/g;x')
        self.checkJoin(RFC3986_BASE, 'g;x?y#s','http://a/b/c/g;x?y#s')
        self.checkJoin(RFC3986_BASE, '','http://a/b/c/d;p?q')
        self.checkJoin(RFC3986_BASE, '.','http://a/b/c/')
        self.checkJoin(RFC3986_BASE, './','http://a/b/c/')
        self.checkJoin(RFC3986_BASE, '..','http://a/b/')
        self.checkJoin(RFC3986_BASE, '../','http://a/b/')
        self.checkJoin(RFC3986_BASE, '../g','http://a/b/g')
        self.checkJoin(RFC3986_BASE, '../..','http://a/')
        self.checkJoin(RFC3986_BASE, '../../','http://a/')
        self.checkJoin(RFC3986_BASE, '../../g','http://a/g')

        #Abnormal Examples

        # The 'abnormal scenarios' are incompatible with RFC2986 parsing
        # Tests are here for reference.

        #self.checkJoin(RFC3986_BASE, '../../../g','http://a/g')
        #self.checkJoin(RFC3986_BASE, '../../../../g','http://a/g')
        #self.checkJoin(RFC3986_BASE, '/./g','http://a/g')
        #self.checkJoin(RFC3986_BASE, '/../g','http://a/g')

        self.checkJoin(RFC3986_BASE, 'g.','http://a/b/c/g.')
        self.checkJoin(RFC3986_BASE, '.g','http://a/b/c/.g')
        self.checkJoin(RFC3986_BASE, 'g..','http://a/b/c/g..')
        self.checkJoin(RFC3986_BASE, '..g','http://a/b/c/..g')
        self.checkJoin(RFC3986_BASE, './../g','http://a/b/g')
        self.checkJoin(RFC3986_BASE, './g/.','http://a/b/c/g/')
        self.checkJoin(RFC3986_BASE, 'g/./h','http://a/b/c/g/h')
        self.checkJoin(RFC3986_BASE, 'g/../h','http://a/b/c/h')
        self.checkJoin(RFC3986_BASE, 'g;x=1/./y','http://a/b/c/g;x=1/y')
        self.checkJoin(RFC3986_BASE, 'g;x=1/../y','http://a/b/c/y')
        self.checkJoin(RFC3986_BASE, 'g?y/./x','http://a/b/c/g?y/./x')
        self.checkJoin(RFC3986_BASE, 'g?y/../x','http://a/b/c/g?y/../x')
        self.checkJoin(RFC3986_BASE, 'g#s/./x','http://a/b/c/g#s/./x')
        self.checkJoin(RFC3986_BASE, 'g#s/../x','http://a/b/c/g#s/../x')
        #self.checkJoin(RFC3986_BASE, 'http:g','http:g') # strict parser
        self.checkJoin(RFC3986_BASE, 'http:g','http://a/b/c/g') # relaxed parser

        # Test for issue9721
        self.checkJoin('http://a/b/c/de', ';x','http://a/b/c/;x')

    def test_urljoins(self):
        self.checkJoin(SIMPLE_BASE, 'g:h','g:h')
        self.checkJoin(SIMPLE_BASE, 'http:g','http://a/b/c/g')
        self.checkJoin(SIMPLE_BASE, 'http:','http://a/b/c/d')
        self.checkJoin(SIMPLE_BASE, 'g','http://a/b/c/g')
        self.checkJoin(SIMPLE_BASE, './g','http://a/b/c/g')
        self.checkJoin(SIMPLE_BASE, 'g/','http://a/b/c/g/')
        self.checkJoin(SIMPLE_BASE, '/g','http://a/g')
        self.checkJoin(SIMPLE_BASE, '//g','http://g')
        self.checkJoin(SIMPLE_BASE, '?y','http://a/b/c/d?y')
        self.checkJoin(SIMPLE_BASE, 'g?y','http://a/b/c/g?y')
        self.checkJoin(SIMPLE_BASE, 'g?y/./x','http://a/b/c/g?y/./x')
        self.checkJoin(SIMPLE_BASE, '.','http://a/b/c/')
        self.checkJoin(SIMPLE_BASE, './','http://a/b/c/')
        self.checkJoin(SIMPLE_BASE, '..','http://a/b/')
        self.checkJoin(SIMPLE_BASE, '../','http://a/b/')
        self.checkJoin(SIMPLE_BASE, '../g','http://a/b/g')
        self.checkJoin(SIMPLE_BASE, '../..','http://a/')
        self.checkJoin(SIMPLE_BASE, '../../g','http://a/g')
        self.checkJoin(SIMPLE_BASE, '../../../g','http://a/../g')
        self.checkJoin(SIMPLE_BASE, './../g','http://a/b/g')
        self.checkJoin(SIMPLE_BASE, './g/.','http://a/b/c/g/')
        self.checkJoin(SIMPLE_BASE, '/./g','http://a/./g')
        self.checkJoin(SIMPLE_BASE, 'g/./h','http://a/b/c/g/h')
        self.checkJoin(SIMPLE_BASE, 'g/../h','http://a/b/c/h')
        self.checkJoin(SIMPLE_BASE, 'http:g','http://a/b/c/g')
        self.checkJoin(SIMPLE_BASE, 'http:','http://a/b/c/d')
        self.checkJoin(SIMPLE_BASE, 'http:?y','http://a/b/c/d?y')
        self.checkJoin(SIMPLE_BASE, 'http:g?y','http://a/b/c/g?y')
        self.checkJoin(SIMPLE_BASE, 'http:g?y/./x','http://a/b/c/g?y/./x')
        self.checkJoin('http:///', '..','http:///')
        self.checkJoin('', 'http://a/b/c/g?y/./x','http://a/b/c/g?y/./x')
        self.checkJoin('', 'http://a/./g', 'http://a/./g')
        self.checkJoin('svn://pathtorepo/dir1','dir2','svn://pathtorepo/dir2')
        self.checkJoin('svn+ssh://pathtorepo/dir1','dir2','svn+ssh://pathtorepo/dir2')

    def test_RFC2732(self):
        for url, hostname, port in [
            ('http://Test.python.org:5432/foo/', 'test.python.org', 5432),
            ('http://12.34.56.78:5432/foo/', '12.34.56.78', 5432),
            ('http://[::1]:5432/foo/', '::1', 5432),
            ('http://[dead:beef::1]:5432/foo/', 'dead:beef::1', 5432),
            ('http://[dead:beef::]:5432/foo/', 'dead:beef::', 5432),
            ('http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]:5432/foo/',
             'dead:beef:cafe:5417:affe:8fa3:deaf:feed', 5432),
            ('http://[::12.34.56.78]:5432/foo/', '::12.34.56.78', 5432),
            ('http://[::ffff:12.34.56.78]:5432/foo/',
             '::ffff:12.34.56.78', 5432),
            ('http://Test.python.org/foo/', 'test.python.org', None),
            ('http://12.34.56.78/foo/', '12.34.56.78', None),
            ('http://[::1]/foo/', '::1', None),
            ('http://[dead:beef::1]/foo/', 'dead:beef::1', None),
            ('http://[dead:beef::]/foo/', 'dead:beef::', None),
            ('http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]/foo/',
             'dead:beef:cafe:5417:affe:8fa3:deaf:feed', None),
            ('http://[::12.34.56.78]/foo/', '::12.34.56.78', None),
            ('http://[::ffff:12.34.56.78]/foo/',
             '::ffff:12.34.56.78', None),
            ('http://Test.python.org:/foo/', 'test.python.org', None),
            ('http://12.34.56.78:/foo/', '12.34.56.78', None),
            ('http://[::1]:/foo/', '::1', None),
            ('http://[dead:beef::1]:/foo/', 'dead:beef::1', None),
            ('http://[dead:beef::]:/foo/', 'dead:beef::', None),
            ('http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]:/foo/',
             'dead:beef:cafe:5417:affe:8fa3:deaf:feed', None),
            ('http://[::12.34.56.78]:/foo/', '::12.34.56.78', None),
            ('http://[::ffff:12.34.56.78]:/foo/',
             '::ffff:12.34.56.78', None),
            ]:
            urlparsed = urlparse.urlparse(url)
            self.assertEqual((urlparsed.hostname, urlparsed.port) , (hostname, port))

        for invalid_url in [
                'http://::12.34.56.78]/',
                'http://[::1/foo/',
                'ftp://[::1/foo/bad]/bad',
                'http://[::1/foo/bad]/bad',
                'http://[::ffff:12.34.56.78']:
            self.assertRaises(ValueError, urlparse.urlparse, invalid_url)

    def test_urldefrag(self):
        for url, defrag, frag in [
            ('http://python.org#frag', 'http://python.org', 'frag'),
            ('http://python.org', 'http://python.org', ''),
            ('http://python.org/#frag', 'http://python.org/', 'frag'),
            ('http://python.org/', 'http://python.org/', ''),
            ('http://python.org/?q#frag', 'http://python.org/?q', 'frag'),
            ('http://python.org/?q', 'http://python.org/?q', ''),
            ('http://python.org/p#frag', 'http://python.org/p', 'frag'),
            ('http://python.org/p?q', 'http://python.org/p?q', ''),
            (RFC1808_BASE, 'http://a/b/c/d;p?q', 'f'),
            (RFC2396_BASE, 'http://a/b/c/d;p?q', ''),
            ]:
            self.assertEqual(urlparse.urldefrag(url), (defrag, frag))

    def test_urlsplit_attributes(self):
        url = "HTTP://WWW.PYTHON.ORG/doc/#frag"
        p = urlparse.urlsplit(url)
        self.assertEqual(p.scheme, "http")
        self.assertEqual(p.netloc, "WWW.PYTHON.ORG")
        self.assertEqual(p.path, "/doc/")
        self.assertEqual(p.query, "")
        self.assertEqual(p.fragment, "frag")
        self.assertEqual(p.username, None)
        self.assertEqual(p.password, None)
        self.assertEqual(p.hostname, "www.python.org")
        self.assertEqual(p.port, None)
        # geturl() won't return exactly the original URL in this case
        # since the scheme is always case-normalized
        #self.assertEqual(p.geturl(), url)

        url = "http://User:Pass@www.python.org:080/doc/?query=yes#frag"
        p = urlparse.urlsplit(url)
        self.assertEqual(p.scheme, "http")
        self.assertEqual(p.netloc, "User:Pass@www.python.org:080")
        self.assertEqual(p.path, "/doc/")
        self.assertEqual(p.query, "query=yes")
        self.assertEqual(p.fragment, "frag")
        self.assertEqual(p.username, "User")
        self.assertEqual(p.password, "Pass")
        self.assertEqual(p.hostname, "www.python.org")
        self.assertEqual(p.port, 80)
        self.assertEqual(p.geturl(), url)

        # Addressing issue1698, which suggests Username can contain
        # "@" characters.  Though not RFC compliant, many ftp sites allow
        # and request email addresses as usernames.

        url = "http://User@example.com:Pass@www.python.org:080/doc/?query=yes#frag"
        p = urlparse.urlsplit(url)
        self.assertEqual(p.scheme, "http")
        self.assertEqual(p.netloc, "User@example.com:Pass@www.python.org:080")
        self.assertEqual(p.path, "/doc/")
        self.assertEqual(p.query, "query=yes")
        self.assertEqual(p.fragment, "frag")
        self.assertEqual(p.username, "User@example.com")
        self.assertEqual(p.password, "Pass")
        self.assertEqual(p.hostname, "www.python.org")
        self.assertEqual(p.port, 80)
        self.assertEqual(p.geturl(), url)

        # Verify an illegal port of value greater than 65535 is set as None
        url = "http://www.python.org:65536"
        p = urlparse.urlsplit(url)
        self.assertEqual(p.port, None)

    def test_issue14072(self):
        p1 = urlparse.urlsplit('tel:+31-641044153')
        self.assertEqual(p1.scheme, 'tel')
        self.assertEqual(p1.path, '+31-641044153')

        p2 = urlparse.urlsplit('tel:+31641044153')
        self.assertEqual(p2.scheme, 'tel')
        self.assertEqual(p2.path, '+31641044153')

        # Assert for urlparse
        p1 = urlparse.urlparse('tel:+31-641044153')
        self.assertEqual(p1.scheme, 'tel')
        self.assertEqual(p1.path, '+31-641044153')

        p2 = urlparse.urlparse('tel:+31641044153')
        self.assertEqual(p2.scheme, 'tel')
        self.assertEqual(p2.path, '+31641044153')


    def test_telurl_params(self):
        p1 = urlparse.urlparse('tel:123-4;phone-context=+1-650-516')
        self.assertEqual(p1.scheme, 'tel')
        self.assertEqual(p1.path, '123-4')
        self.assertEqual(p1.params, 'phone-context=+1-650-516')

        p1 = urlparse.urlparse('tel:+1-201-555-0123')
        self.assertEqual(p1.scheme, 'tel')
        self.assertEqual(p1.path, '+1-201-555-0123')
        self.assertEqual(p1.params, '')

        p1 = urlparse.urlparse('tel:7042;phone-context=example.com')
        self.assertEqual(p1.scheme, 'tel')
        self.assertEqual(p1.path, '7042')
        self.assertEqual(p1.params, 'phone-context=example.com')

        p1 = urlparse.urlparse('tel:863-1234;phone-context=+1-914-555')
        self.assertEqual(p1.scheme, 'tel')
        self.assertEqual(p1.path, '863-1234')
        self.assertEqual(p1.params, 'phone-context=+1-914-555')


    def test_urlsplit_remove_unsafe_bytes(self):
        # Remove ASCII tabs and newlines from input
        url = "http://www.python.org/java\nscript:\talert('msg\r\n')/#frag"
        p = urlparse.urlsplit(url)
        self.assertEqual(p.scheme, "http")
        self.assertEqual(p.netloc, "www.python.org")
        self.assertEqual(p.path, "/javascript:alert('msg')/")
        self.assertEqual(p.query, "")
        self.assertEqual(p.fragment, "frag")
        self.assertEqual(p.username, None)
        self.assertEqual(p.password, None)
        self.assertEqual(p.hostname, "www.python.org")
        self.assertEqual(p.port, None)
        self.assertEqual(p.geturl(), "http://www.python.org/javascript:alert('msg')/#frag")

        # Remove ASCII tabs and newlines from input as bytes.
        url = b"http://www.python.org/java\nscript:\talert('msg\r\n')/#frag"
        p = urlparse.urlsplit(url)
        self.assertEqual(p.scheme, b"http")
        self.assertEqual(p.netloc, b"www.python.org")
        self.assertEqual(p.path, b"/javascript:alert('msg')/")
        self.assertEqual(p.query, b"")
        self.assertEqual(p.fragment, b"frag")
        self.assertEqual(p.username, None)
        self.assertEqual(p.password, None)
        self.assertEqual(p.hostname, b"www.python.org")
        self.assertEqual(p.port, None)
        self.assertEqual(p.geturl(), b"http://www.python.org/javascript:alert('msg')/#frag")

    def test_attributes_bad_port(self):
        """Check handling of non-integer ports."""
        p = urlparse.urlsplit("http://www.example.net:foo")
        self.assertEqual(p.netloc, "www.example.net:foo")
        self.assertRaises(ValueError, lambda: p.port)

        p = urlparse.urlparse("http://www.example.net:foo")
        self.assertEqual(p.netloc, "www.example.net:foo")
        self.assertRaises(ValueError, lambda: p.port)

    def test_attributes_without_netloc(self):
        # This example is straight from RFC 3261.  It looks like it
        # should allow the username, hostname, and port to be filled
        # in, but doesn't.  Since it's a URI and doesn't use the
        # scheme://netloc syntax, the netloc and related attributes
        # should be left empty.
        uri = "sip:alice@atlanta.com;maddr=239.255.255.1;ttl=15"
        p = urlparse.urlsplit(uri)
        self.assertEqual(p.netloc, "")
        self.assertEqual(p.username, None)
        self.assertEqual(p.password, None)
        self.assertEqual(p.hostname, None)
        self.assertEqual(p.port, None)
        self.assertEqual(p.geturl(), uri)

        p = urlparse.urlparse(uri)
        self.assertEqual(p.netloc, "")
        self.assertEqual(p.username, None)
        self.assertEqual(p.password, None)
        self.assertEqual(p.hostname, None)
        self.assertEqual(p.port, None)
        self.assertEqual(p.geturl(), uri)

    def test_caching(self):
        # Test case for bug #1313119
        uri = "http://example.com/doc/"
        unicode_uri = unicode(uri)

        urlparse.urlparse(unicode_uri)
        p = urlparse.urlparse(uri)
        self.assertEqual(type(p.scheme), type(uri))
        self.assertEqual(type(p.hostname), type(uri))
        self.assertEqual(type(p.path), type(uri))

    def test_noslash(self):
        # Issue 1637: http://foo.com?query is legal
        self.assertEqual(urlparse.urlparse("http://example.com?blahblah=/foo"),
                         ('http', 'example.com', '', '', 'blahblah=/foo', ''))

    def test_anyscheme(self):
        # Issue 7904: s3://foo.com/stuff has netloc "foo.com".
        self.assertEqual(urlparse.urlparse("s3://foo.com/stuff"),
                         ('s3','foo.com','/stuff','','',''))
        self.assertEqual(urlparse.urlparse("x-newscheme://foo.com/stuff"),
                         ('x-newscheme','foo.com','/stuff','','',''))
        self.assertEqual(urlparse.urlparse("x-newscheme://foo.com/stuff?query#fragment"),
                         ('x-newscheme','foo.com','/stuff','','query','fragment'))
        self.assertEqual(urlparse.urlparse("x-newscheme://foo.com/stuff?query"),
                         ('x-newscheme','foo.com','/stuff','','query',''))

    def test_withoutscheme(self):
        # Test urlparse without scheme
        # Issue 754016: urlparse goes wrong with IP:port without scheme
        # RFC 1808 specifies that netloc should start with //, urlparse expects
        # the same, otherwise it classifies the portion of url as path.
        self.assertEqual(urlparse.urlparse("path"),
                ('','','path','','',''))
        self.assertEqual(urlparse.urlparse("//www.python.org:80"),
                ('','www.python.org:80','','','',''))
        self.assertEqual(urlparse.urlparse("http://www.python.org:80"),
                ('http','www.python.org:80','','','',''))

    def test_portseparator(self):
        # Issue 754016 makes changes for port separator ':' from scheme separator
        self.assertEqual(urlparse.urlparse("path:80"),
                ('','','path:80','','',''))
        self.assertEqual(urlparse.urlparse("http:"),('http','','','','',''))
        self.assertEqual(urlparse.urlparse("https:"),('https','','','','',''))
        self.assertEqual(urlparse.urlparse("http://www.python.org:80"),
                ('http','www.python.org:80','','','',''))

    def test_urlsplit_normalization(self):
        # Certain characters should never occur in the netloc,
        # including under normalization.
        # Ensure that ALL of them are detected and cause an error
        illegal_chars = u'/:#?@'
        hex_chars = {'{:04X}'.format(ord(c)) for c in illegal_chars}
        denorm_chars = [
            c for c in map(unichr, range(128, sys.maxunicode))
            if (hex_chars & set(unicodedata.decomposition(c).split()))
            and c not in illegal_chars
        ]
        # Sanity check that we found at least one such character
        self.assertIn(u'\u2100', denorm_chars)
        self.assertIn(u'\uFF03', denorm_chars)

        # bpo-36742: Verify port separators are ignored when they
        # existed prior to decomposition
        urlparse.urlsplit(u'http://\u30d5\u309a:80')
        with self.assertRaises(ValueError):
            urlparse.urlsplit(u'http://\u30d5\u309a\ufe1380')

        for scheme in [u"http", u"https", u"ftp"]:
            for netloc in [u"netloc{}false.netloc", u"n{}user@netloc"]:
                for c in denorm_chars:
                    url = u"{}://{}/path".format(scheme, netloc.format(c))
                    if test_support.verbose:
                        print "Checking %r" % url
                    with self.assertRaises(ValueError):
                        urlparse.urlsplit(url)

        # check error message: invalid netloc must be formated with repr()
        # to get an ASCII error message
        with self.assertRaises(ValueError) as cm:
            urlparse.urlsplit(u'http://example.com\uFF03@bing.com')
        self.assertEqual(str(cm.exception),
                         "netloc u'example.com\\uff03@bing.com' contains invalid characters "
                         "under NFKC normalization")
        self.assertIsInstance(cm.exception.args[0], str)

def test_main():
    test_support.run_unittest(UrlParseTestCase)

if __name__ == "__main__":
    test_main()



"""Suite Files: Classes representing files
Level 1, version 1

Generated from /System/Library/CoreServices/Finder.app
AETE/AEUT resource version 0/144, language 0, script 0
"""

import aetools
import MacOS

_code = 'fndr'

class Files_Events:

    pass


class alias_file(aetools.ComponentItem):
    """alias file - An alias file (created with \xd2Make Alias\xd3) """
    want = 'alia'
class _Prop__3c_Inheritance_3e_(aetools.NProperty):
    """<Inheritance> - inherits some of its properties from the file class """
    which = 'c@#^'
    want = 'file'
class _Prop_original_item(aetools.NProperty):
    """original item - the original item pointed to by the alias """
    which = 'orig'
    want = 'obj '

alias_files = alias_file

class application_file(aetools.ComponentItem):
    """application file - An application's file on disk """
    want = 'appf'
class _Prop_accepts_high_level_events(aetools.NProperty):
    """accepts high level events - Is the application high-level event aware? (OBSOLETE: always returns true) """
    which = 'isab'
    want = 'bool'
class _Prop_has_scripting_terminology(aetools.NProperty):
    """has scripting terminology - Does the process have a scripting terminology, i.e., can it be scripted? """
    which = 'hscr'
    want = 'bool'
class _Prop_minimum_size(aetools.NProperty):
    """minimum size - the smallest memory size with which the application can be launched """
    which = 'mprt'
    want = 'long'
class _Prop_opens_in_Classic(aetools.NProperty):
    """opens in Classic - Should the application launch in the Classic environment? """
    which = 'Clsc'
    want = 'bool'
class _Prop_preferred_size(aetools.NProperty):
    """preferred size - the memory size with which the application will be launched """
    which = 'appt'
    want = 'long'
class _Prop_suggested_size(aetools.NProperty):
    """suggested size - the memory size with which the developer recommends the application be launched """
    which = 'sprt'
    want = 'long'

application_files = application_file

class clipping(aetools.ComponentItem):
    """clipping - A clipping """
    want = 'clpf'
class _Prop_clipping_window(aetools.NProperty):
    """clipping window - (NOT AVAILABLE YET) the clipping window for this clipping """
    which = 'lwnd'
    want = 'obj '

clippings = clipping

class document_file(aetools.ComponentItem):
    """document file - A document file """
    want = 'docf'

document_files = document_file

class file(aetools.ComponentItem):
    """file - A file """
    want = 'file'
class _Prop_creator_type(aetools.NProperty):
    """creator type - the OSType identifying the application that created the item """
    which = 'fcrt'
    want = 'type'
class _Prop_file_type(aetools.NProperty):
    """file type - the OSType identifying the type of data contained in the item """
    which = 'asty'
    want = 'type'
class _Prop_product_version(aetools.NProperty):
    """product version - the version of the product (visible at the top of the \xd2Get Info\xd3 window) """
    which = 'ver2'
    want = 'utxt'
class _Prop_stationery(aetools.NProperty):
    """stationery - Is the file a stationery pad? """
    which = 'pspd'
    want = 'bool'
class _Prop_version(aetools.NProperty):
    """version - the version of the file (visible at the bottom of the \xd2Get Info\xd3 window) """
    which = 'vers'
    want = 'utxt'

files = file

class internet_location_file(aetools.ComponentItem):
    """internet location file - A file containing an internet location """
    want = 'inlf'
class _Prop_location(aetools.NProperty):
    """location - the internet location """
    which = 'iloc'
    want = 'utxt'

internet_location_files = internet_location_file

class package(aetools.ComponentItem):
    """package - A package """
    want = 'pack'

packages = package
alias_file._superclassnames = ['file']
alias_file._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
    'original_item' : _Prop_original_item,
}
alias_file._privelemdict = {
}
application_file._superclassnames = ['file']
application_file._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
    'accepts_high_level_events' : _Prop_accepts_high_level_events,
    'has_scripting_terminology' : _Prop_has_scripting_terminology,
    'minimum_size' : _Prop_minimum_size,
    'opens_in_Classic' : _Prop_opens_in_Classic,
    'preferred_size' : _Prop_preferred_size,
    'suggested_size' : _Prop_suggested_size,
}
application_file._privelemdict = {
}
clipping._superclassnames = ['file']
clipping._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
    'clipping_window' : _Prop_clipping_window,
}
clipping._privelemdict = {
}
document_file._superclassnames = ['file']
document_file._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
}
document_file._privelemdict = {
}
import Finder_items
file._superclassnames = ['item']
file._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
    'creator_type' : _Prop_creator_type,
    'file_type' : _Prop_file_type,
    'product_version' : _Prop_product_version,
    'stationery' : _Prop_stationery,
    'version' : _Prop_version,
}
file._privelemdict = {
}
internet_location_file._superclassnames = ['file']
internet_location_file._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
    'location' : _Prop_location,
}
internet_location_file._privelemdict = {
}
package._superclassnames = ['item']
package._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
}
package._privelemdict = {
}

#
# Indices of types declared in this module
#
_classdeclarations = {
    'alia' : alias_file,
    'appf' : application_file,
    'clpf' : clipping,
    'docf' : document_file,
    'file' : file,
    'inlf' : internet_location_file,
    'pack' : package,
}

_propdeclarations = {
    'Clsc' : _Prop_opens_in_Classic,
    'appt' : _Prop_preferred_size,
    'asty' : _Prop_file_type,
    'c@#^' : _Prop__3c_Inheritance_3e_,
    'fcrt' : _Prop_creator_type,
    'hscr' : _Prop_has_scripting_terminology,
    'iloc' : _Prop_location,
    'isab' : _Prop_accepts_high_level_events,
    'lwnd' : _Prop_clipping_window,
    'mprt' : _Prop_minimum_size,
    'orig' : _Prop_original_item,
    'pspd' : _Prop_stationery,
    'sprt' : _Prop_suggested_size,
    'ver2' : _Prop_product_version,
    'vers' : _Prop_version,
}

_compdeclarations = {
}

_enumdeclarations = {
}



import unittest
import Tkinter as tkinter
from test.test_support import requires, run_unittest
from test_ttk.support import AbstractTkTest

requires('gui')

class TextTest(AbstractTkTest, unittest.TestCase):

    def setUp(self):
        super(TextTest, self).setUp()
        self.text = tkinter.Text(self.root)

    def test_debug(self):
        text = self.text
        olddebug = text.debug()
        try:
            text.debug(0)
            self.assertEqual(text.debug(), 0)
            text.debug(1)
            self.assertEqual(text.debug(), 1)
        finally:
            text.debug(olddebug)
            self.assertEqual(text.debug(), olddebug)

    def test_search(self):
        text = self.text

        # pattern and index are obligatory arguments.
        self.assertRaises(tkinter.TclError, text.search, None, '1.0')
        self.assertRaises(tkinter.TclError, text.search, 'a', None)
        self.assertRaises(tkinter.TclError, text.search, None, None)

        # Invalid text index.
        self.assertRaises(tkinter.TclError, text.search, '', 0)

        # Check if we are getting the indices as strings -- you are likely
        # to get Tcl_Obj under Tk 8.5 if Tkinter doesn't convert it.
        text.insert('1.0', 'hi-test')
        self.assertEqual(text.search('-test', '1.0', 'end'), '1.2')
        self.assertEqual(text.search('test', '1.0', 'end'), '1.3')


tests_gui = (TextTest, )

if __name__ == "__main__":
    run_unittest(*tests_gui)



"""Suite AppleScript Suite: Standard terms for AppleScript
Level 1, version 1

Generated from /Volumes/Sap/System Folder/Extensions/AppleScript
AETE/AEUT resource version 1/0, language 0, script 0
"""

import aetools
import MacOS

_code = 'ascr'

class AppleScript_Suite_Events:

    def _26_(self, _object, _attributes={}, **_arguments):
        """&: Concatenation
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'ccat'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _2a_(self, _object, _attributes={}, **_arguments):
        """*: Multiplication
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '*   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _2b_(self, _object, _attributes={}, **_arguments):
        """+: Addition
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '+   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _2d_(self, _object, _attributes={}, **_arguments):
        """-: Subtraction
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '-   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _3c_(self, _object, _attributes={}, **_arguments):
        """<: Less than
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '<   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _3d_(self, _object, _attributes={}, **_arguments):
        """=: Equality
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '=   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _3e_(self, _object, _attributes={}, **_arguments):
        """>: Greater than
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '>   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_Call_a5_subroutine = {
        'at' : 'at  ',
        'from_' : 'from',
        'for_' : 'for ',
        'to' : 'to  ',
        'thru' : 'thru',
        'through' : 'thgh',
        'by' : 'by  ',
        'on' : 'on  ',
        'into' : 'into',
        'onto' : 'onto',
        'between' : 'btwn',
        'against' : 'agst',
        'out_of' : 'outo',
        'instead_of' : 'isto',
        'aside_from' : 'asdf',
        'around' : 'arnd',
        'beside' : 'bsid',
        'beneath' : 'bnth',
        'under' : 'undr',
        'over' : 'over',
        'above' : 'abve',
        'below' : 'belw',
        'apart_from' : 'aprt',
        'about' : 'abou',
        'since' : 'snce',
        'given' : 'givn',
        'with' : 'with',
        'without' : 'wout',
    }

    def Call_a5_subroutine(self, _object=None, _attributes={}, **_arguments):
        """Call\xa5subroutine: A subroutine call
        Required argument: anything
        Keyword argument at: a preposition
        Keyword argument from_: a preposition
        Keyword argument for_: a preposition
        Keyword argument to: a preposition
        Keyword argument thru: a preposition
        Keyword argument through: a preposition
        Keyword argument by: a preposition
        Keyword argument on: a preposition
        Keyword argument into: a preposition
        Keyword argument onto: a preposition
        Keyword argument between: a preposition
        Keyword argument against: a preposition
        Keyword argument out_of: a preposition
        Keyword argument instead_of: a preposition
        Keyword argument aside_from: a preposition
        Keyword argument around: a preposition
        Keyword argument beside: a preposition
        Keyword argument beneath: a preposition
        Keyword argument under: a preposition
        Keyword argument over: a preposition
        Keyword argument above: a preposition
        Keyword argument below: a preposition
        Keyword argument apart_from: a preposition
        Keyword argument about: a preposition
        Keyword argument since: a preposition
        Keyword argument given: parameter:value pairs, comma-separated
        Keyword argument with: formal parameter set to true if matching actual parameter is provided
        Keyword argument without: formal parameter set to false if matching actual parmeter is provided
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'psbr'

        aetools.keysubst(_arguments, self._argmap_Call_a5_subroutine)
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _5e_(self, _object, _attributes={}, **_arguments):
        """^: Exponentiation
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '^   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def activate(self, _no_object=None, _attributes={}, **_arguments):
        """activate: Bring the targeted application program to the front
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'misc'
        _subcode = 'actv'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def and_(self, _object, _attributes={}, **_arguments):
        """and: Logical conjunction
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'AND '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def as_(self, _object, _attributes={}, **_arguments):
        """as: Coercion
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'coer'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def contains(self, _object, _attributes={}, **_arguments):
        """contains: Containment
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'cont'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def div(self, _object, _attributes={}, **_arguments):
        """div: Quotient
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'div '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def end_tell(self, _no_object=None, _attributes={}, **_arguments):
        """end tell: Record or log an \xd4end tell\xd5 statement
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ascr'
        _subcode = 'tend'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def ends_with(self, _object, _attributes={}, **_arguments):
        """ends with: Ends with
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'ends'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_error = {
        'number' : 'errn',
        'partial_result' : 'ptlr',
        'from_' : 'erob',
        'to' : 'errt',
    }

    def error(self, _object=None, _attributes={}, **_arguments):
        """error: Raise an error
        Required argument: anything
        Keyword argument number: an error number
        Keyword argument partial_result: any partial result occurring before the error
        Keyword argument from_: the object that caused the error
        Keyword argument to: the desired class for a failed coercion
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ascr'
        _subcode = 'err '

        aetools.keysubst(_arguments, self._argmap_error)
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def idle(self, _no_object=None, _attributes={}, **_arguments):
        """idle: Sent to a script application when it is idle
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: the number of seconds to wait for next idle event
        """
        _code = 'misc'
        _subcode = 'idle'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def launch(self, _no_object=None, _attributes={}, **_arguments):
        """launch: Start an application for scripting
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ascr'
        _subcode = 'noop'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def log(self, _object, _attributes={}, **_arguments):
        """log: Cause a comment to be logged
        Required argument: undocumented, typecode 'TEXT'
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ascr'
        _subcode = 'cmnt'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def mod(self, _object, _attributes={}, **_arguments):
        """mod: Remainder
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'mod '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def negate(self, _object, _attributes={}, **_arguments):
        """negate: Numeric negation
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'neg '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def not_(self, _object, _attributes={}, **_arguments):
        """not: Logical negation
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'NOT '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def or_(self, _object, _attributes={}, **_arguments):
        """or: Logical disjunction
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'OR  '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def start_log(self, _no_object=None, _attributes={}, **_arguments):
        """start log: Start event logging in the script editor
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ToyS'
        _subcode = 'log1'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def starts_with(self, _object, _attributes={}, **_arguments):
        """starts with: Starts with
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = 'bgwt'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def stop_log(self, _no_object=None, _attributes={}, **_arguments):
        """stop log: Stop event logging in the script editor
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ToyS'
        _subcode = 'log0'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def tell(self, _no_object=None, _attributes={}, **_arguments):
        """tell: Record or log a \xd4tell\xd5 statement
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ascr'
        _subcode = 'tell'

        if _arguments: raise TypeError, 'No optional args expected'
        if _no_object is not None: raise TypeError, 'No direct arg expected'


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _ad_(self, _object, _attributes={}, **_arguments):
        """\xad: Inequality
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '\xad   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _b2_(self, _object, _attributes={}, **_arguments):
        """\xb2: Less than or equal to
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '<=  '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _b3_(self, _object, _attributes={}, **_arguments):
        """\xb3: Greater than or equal to
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '>=  '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def _d6_(self, _object, _attributes={}, **_arguments):
        """\xd6: Division
        Required argument: an AE object reference
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: anything
        """
        _code = 'ascr'
        _subcode = '/   '

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                _arguments, _attributes)
        if _arguments.get('errn', 0):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']


class anything(aetools.ComponentItem):
    """anything - any class or reference """
    want = '****'

class pictures(aetools.ComponentItem):
    """pictures -  """
    want = 'PICT'

picture = pictures

class styled_text(aetools.ComponentItem):
    """styled text - text with font, size, and style information """
    want = 'STXT'

styled_text = styled_text

class strings(aetools.ComponentItem):
    """strings -  """
    want = 'TEXT'

string = strings

class alias(aetools.ComponentItem):
    """alias - a file on a disk or server.  The file must exist when you check the syntax of your script. """
    want = 'alis'
class _Prop_POSIX_path(aetools.NProperty):
    """POSIX path - the POSIX path of the file """
    which = 'psxp'
    want = 'TEXT'

aliases = alias

class April(aetools.ComponentItem):
    """April - the month of April """
    want = 'apr '

class August(aetools.ComponentItem):
    """August - the month of August """
    want = 'aug '

class booleans(aetools.ComponentItem):
    """booleans -  """
    want = 'bool'

boolean = booleans

class RGB_colors(aetools.ComponentItem):
    """RGB colors -  """
    want = 'cRGB'

RGB_color = RGB_colors

class application(aetools.ComponentItem):
    """application - specifies global properties of AppleScript """
    want = 'capp'
class _Prop_AppleScript(aetools.NProperty):
    """AppleScript - the top-level script object """
    which = 'ascr'
    want = 'scpt'
AppleScript = _Prop_AppleScript()
class _Prop_days(aetools.NProperty):
    """days - the number of seconds in a day """
    which = 'days'
    want = 'long'
days = _Prop_days()
class _Prop_hours(aetools.NProperty):
    """hours - the number of seconds in an hour """
    which = 'hour'
    want = 'long'
hours = _Prop_hours()
class _Prop_minutes(aetools.NProperty):
    """minutes - the number of seconds in a minute """
    which = 'min '
    want = 'long'
minutes = _Prop_minutes()
class _Prop_pi(aetools.NProperty):
    """pi - the constant pi """
    which = 'pi  '
    want = 'doub'
pi = _Prop_pi()
class _Prop_print_depth(aetools.NProperty):
    """print depth - the maximum depth to print """
    which = 'prdp'
    want = 'long'
print_depth = _Prop_print_depth()
class _Prop_print_length(aetools.NProperty):
    """print length - the maximum length to print """
    which = 'prln'
    want = 'long'
print_length = _Prop_print_length()
class _Prop_result(aetools.NProperty):
    """result - the last result of evaluation """
    which = 'rslt'
    want = '****'
result = _Prop_result()
class _Prop_return_(aetools.NProperty):
    """return - a return character """
    which = 'ret '
    want = 'cha '
return_ = _Prop_return_()
class _Prop_space(aetools.NProperty):
    """space - a space character """
    which = 'spac'
    want = 'cha '
space = _Prop_space()
class _Prop_tab(aetools.NProperty):
    """tab - a tab character """
    which = 'tab '
    want = 'cha '
tab = _Prop_tab()
class _Prop_text_item_delimiters(aetools.NProperty):
    """text item delimiters - the text item delimiters of a string """
    which = 'txdl'
    want = 'list'
text_item_delimiters = _Prop_text_item_delimiters()
class _Prop_weeks(aetools.NProperty):
    """weeks - the number of seconds in a week """
    which = 'week'
    want = 'long'
weeks = _Prop_weeks()

applications = application

app = application

class upper_case(aetools.ComponentItem):
    """upper case - Text with lower case converted to upper case """
    want = 'case'

class cubic_centimeters(aetools.ComponentItem):
    """cubic centimeters - a volume measurement in SI cubic centimeters """
    want = 'ccmt'

cubic_centimetres = cubic_centimeters

class cubic_feet(aetools.ComponentItem):
    """cubic feet - a volume measurement in Imperial cubic feet """
    want = 'cfet'

class characters(aetools.ComponentItem):
    """characters -  """
    want = 'cha '

character = characters

class writing_code_info(aetools.ComponentItem):
    """writing code info - script code and language code of text run """
    want = 'citl'
class _Prop_language_code(aetools.NProperty):
    """language code - the language code for the text """
    which = 'plcd'
    want = 'shor'
class _Prop_script_code(aetools.NProperty):
    """script code - the script code for the text """
    which = 'pscd'
    want = 'shor'

writing_code_infos = writing_code_info

class text_items(aetools.ComponentItem):
    """text items -  """
    want = 'citm'

text_item = text_items

class cubic_meters(aetools.ComponentItem):
    """cubic meters - a volume measurement in SI cubic meters """
    want = 'cmet'

cubic_metres = cubic_meters

class centimeters(aetools.ComponentItem):
    """centimeters - a distance measurement in SI centimeters """
    want = 'cmtr'

centimetres = centimeters

class item(aetools.ComponentItem):
    """item - An item of any type """
    want = 'cobj'
class _Prop_id(aetools.NProperty):
    """id - the unique ID number of this object """
    which = 'ID  '
    want = 'long'

items = item

class C_strings(aetools.ComponentItem):
    """C strings -  """
    want = 'cstr'

C_string = C_strings

class text(aetools.ComponentItem):
    """text - text with language and style information """
    want = 'ctxt'

class cubic_inches(aetools.ComponentItem):
    """cubic inches - a volume measurement in Imperial cubic inches """
    want = 'cuin'

class cubic_yards(aetools.ComponentItem):
    """cubic yards - a distance measurement in Imperial cubic yards """
    want = 'cyrd'

class December(aetools.ComponentItem):
    """December - the month of December """
    want = 'dec '

class degrees_Celsius(aetools.ComponentItem):
    """degrees Celsius - a temperature measurement in SI degrees Celsius """
    want = 'degc'

class degrees_Fahrenheit(aetools.ComponentItem):
    """degrees Fahrenheit - a temperature measurement in degrees Fahrenheit """
    want = 'degf'

class degrees_Kelvin(aetools.ComponentItem):
    """degrees Kelvin - a temperature measurement in degrees Kelvin """
    want = 'degk'

class reals(aetools.ComponentItem):
    """reals -  """
    want = 'doub'

real = reals

class encoded_strings(aetools.ComponentItem):
    """encoded strings -  """
    want = 'encs'

encoded_string = encoded_strings

class constants(aetools.ComponentItem):
    """constants -  """
    want = 'enum'

constant = constants

class events(aetools.ComponentItem):
    """events -  """
    want = 'evnt'

event = events

class February(aetools.ComponentItem):
    """February - the month of February """
    want = 'feb '

class feet(aetools.ComponentItem):
    """feet - a distance measurement in Imperial feet """
    want = 'feet'

class Friday(aetools.ComponentItem):
    """Friday - Friday """
    want = 'fri '

class file_specification(aetools.ComponentItem):
    """file specification - a file specification as used by the operating system """
    want = 'fss '

file_specifications = file_specification

class gallons(aetools.ComponentItem):
    """gallons - a volume measurement in Imperial gallons """
    want = 'galn'

class grams(aetools.ComponentItem):
    """grams - a mass measurement in SI meters """
    want = 'gram'

class handlers(aetools.ComponentItem):
    """handlers -  """
    want = 'hand'

handler = handlers

class inches(aetools.ComponentItem):
    """inches - a distance measurement in Imperial inches """
    want = 'inch'

class international_text(aetools.ComponentItem):
    """international text - text that begins with a writing code """
    want = 'itxt'

international_text = international_text

class January(aetools.ComponentItem):
    """January - the month of January """
    want = 'jan '

class July(aetools.ComponentItem):
    """July - the month of July """
    want = 'jul '

class June(aetools.ComponentItem):
    """June - the month of June """
    want = 'jun '

class reference_forms(aetools.ComponentItem):
    """reference forms -  """
    want = 'kfrm'

reference_form = reference_forms

class kilograms(aetools.ComponentItem):
    """kilograms - a mass measurement in SI kilograms """
    want = 'kgrm'

class kilometers(aetools.ComponentItem):
    """kilometers - a distance measurement in SI kilometers """
    want = 'kmtr'

kilometres = kilometers

class keystroke(aetools.ComponentItem):
    """keystroke - a press of a key combination on a Macintosh keyboard """
    want = 'kprs'
class _Prop_key(aetools.NProperty):
    """key - the character for the key was pressed (ignoring modifiers) """
    which = 'kMsg'
    want = 'cha '
class _Prop_key_kind(aetools.NProperty):
    """key kind - the kind of key that was pressed """
    which = 'kknd'
    want = 'ekst'
class _Prop_modifiers(aetools.NProperty):
    """modifiers - the modifier keys pressed in combination """
    which = 'kMod'
    want = 'eMds'

keystrokes = keystroke

class pounds(aetools.ComponentItem):
    """pounds - a weight measurement in SI meters """
    want = 'lbs '

class date(aetools.ComponentItem):
    """date - Absolute date and time values """
    want = 'ldt '
class _Prop_date_string(aetools.NProperty):
    """date string - the date portion of a date-time value as text """
    which = 'dstr'
    want = 'TEXT'
class _Prop_day(aetools.NProperty):
    """day - the day of the month of a date """
    which = 'day '
    want = 'long'
class _Prop_month(aetools.NProperty):
    """month - the month of a date """
    which = 'mnth'
    want = 'mnth'
class _Prop_time(aetools.NProperty):
    """time - the time since midnight of a date """
    which = 'time'
    want = 'long'
class _Prop_time_string(aetools.NProperty):
    """time string - the time portion of a date-time value as text """
    which = 'tstr'
    want = 'TEXT'
class _Prop_weekday(aetools.NProperty):
    """weekday - the day of a week of a date """
    which = 'wkdy'
    want = 'wkdy'
class _Prop_year(aetools.NProperty):
    """year - the year of a date """
    which = 'year'
    want = 'long'

dates = date

class list(aetools.ComponentItem):
    """list - An ordered collection of items """
    want = 'list'
class _Prop_length(aetools.NProperty):
    """length - the length of a list """
    which = 'leng'
    want = 'long'
class _Prop_rest(aetools.NProperty):
    """rest - all items of the list excluding first """
    which = 'rest'
    want = 'list'
class _Prop_reverse(aetools.NProperty):
    """reverse - the items of the list in reverse order """
    which = 'rvse'
    want = 'list'

lists = list

class liters(aetools.ComponentItem):
    """liters - a volume measurement in SI liters """
    want = 'litr'

litres = liters

class linked_list(aetools.ComponentItem):
    """linked list - An ordered collection of items """
    want = 'llst'

linked_lists = linked_list

class integers(aetools.ComponentItem):
    """integers -  """
    want = 'long'

integer = integers

class list_or_record(aetools.ComponentItem):
    """list or record - a list or record """
    want = 'lr  '

class list_2c__record_or_text(aetools.ComponentItem):
    """list, record or text - a list, record or text """
    want = 'lrs '

class list_or_string(aetools.ComponentItem):
    """list or string - a list or string """
    want = 'ls  '

class machines(aetools.ComponentItem):
    """machines -  """
    want = 'mach'

machine = machines

class March(aetools.ComponentItem):
    """March - the month of March """
    want = 'mar '

class May(aetools.ComponentItem):
    """May - the month of May """
    want = 'may '

class meters(aetools.ComponentItem):
    """meters - a distance measurement in SI meters """
    want = 'metr'

metres = meters

class miles(aetools.ComponentItem):
    """miles - a distance measurement in Imperial miles """
    want = 'mile'

class months(aetools.ComponentItem):
    """months -  """
    want = 'mnth'

month = months

class Monday(aetools.ComponentItem):
    """Monday - Monday """
    want = 'mon '

class missing_values(aetools.ComponentItem):
    """missing values -  """
    want = 'msng'

missing_value = missing_values

class number_or_date(aetools.ComponentItem):
    """number or date - a number or date """
    want = 'nd  '

class number_2c__date_or_text(aetools.ComponentItem):
    """number, date or text - a number, date or text """
    want = 'nds '

class numbers(aetools.ComponentItem):
    """numbers -  """
    want = 'nmbr'

number = numbers

class November(aetools.ComponentItem):
    """November - the month of November """
    want = 'nov '

class number_or_string(aetools.ComponentItem):
    """number or string - a number or string """
    want = 'ns  '

class references(aetools.ComponentItem):
    """references -  """
    want = 'obj '

reference = references

class October(aetools.ComponentItem):
    """October - the month of October """
    want = 'oct '

class ounces(aetools.ComponentItem):
    """ounces - a weight measurement in SI meters """
    want = 'ozs '

class class_(aetools.ComponentItem):
    """class - the type of a value """
    want = 'pcls'
class _Prop__3c_Inheritance_3e_(aetools.NProperty):
    """<Inheritance> - inherits some of its properties from this class """
    which = 'c@#^'
    want = 'type'

classes = class_

class prepositions(aetools.ComponentItem):
    """prepositions -  """
    want = 'prep'

preposition = prepositions

class properties(aetools.ComponentItem):
    """properties -  """
    want = 'prop'

property = properties

class writing_code(aetools.ComponentItem):
    """writing code - codes that identify the language and script system """
    want = 'psct'

class Pascal_strings(aetools.ComponentItem):
    """Pascal strings -  """
    want = 'pstr'

Pascal_string = Pascal_strings

class quarts(aetools.ComponentItem):
    """quarts - a volume measurement in Imperial quarts """
    want = 'qrts'

class data(aetools.ComponentItem):
    """data - an AppleScript raw data object """
    want = 'rdat'

class records(aetools.ComponentItem):
    """records -  """
    want = 'reco'

record = records

class Saturday(aetools.ComponentItem):
    """Saturday - Saturday """
    want = 'sat '

class seconds(aetools.ComponentItem):
    """seconds - more than one second """
    want = 'scnd'

class script(aetools.ComponentItem):
    """script - An AppleScript script """
    want = 'scpt'
class _Prop_name(aetools.NProperty):
    """name - the name of the script """
    which = 'pnam'
    want = 'TEXT'
class _Prop_parent(aetools.NProperty):
    """parent - its parent, i.e. the script that will handle events that this script doesn\xd5t """
    which = 'pare'
    want = 'scpt'

scripts = script

class September(aetools.ComponentItem):
    """September - the month of September """
    want = 'sep '

class alias_or_string(aetools.ComponentItem):
    """alias or string - an alias or string """
    want = 'sf  '

class sounds(aetools.ComponentItem):
    """sounds -  """
    want = 'snd '

sound = sounds

class square_feet(aetools.ComponentItem):
    """square feet - an area measurement in Imperial square feet """
    want = 'sqft'

class square_kilometers(aetools.ComponentItem):
    """square kilometers - an area measurement in SI square kilometers """
    want = 'sqkm'

square_kilometres = square_kilometers

class square_miles(aetools.ComponentItem):
    """square miles - an area measurement in Imperial square miles """
    want = 'sqmi'

class square_meters(aetools.ComponentItem):
    """square meters - an area measurement in SI square meters """
    want = 'sqrm'

square_metres = square_meters

class square_yards(aetools.ComponentItem):
    """square yards - an area measurement in Imperial square yards """
    want = 'sqyd'

class styled_Clipboard_text(aetools.ComponentItem):
    """styled Clipboard text - clipboard text with font, size, and style information """
    want = 'styl'

styled_Clipboard_text = styled_Clipboard_text

class Sunday(aetools.ComponentItem):
    """Sunday - Sunday """
    want = 'sun '

class styled_Unicode_text(aetools.ComponentItem):
    """styled Unicode text - styled text in the Unicode format """
    want = 'sutx'

styled_Unicode_text = styled_Unicode_text

class Thursday(aetools.ComponentItem):
    """Thursday - Thursday """
    want = 'thu '

class Tuesday(aetools.ComponentItem):
    """Tuesday - Tuesday """
    want = 'tue '

class type_class(aetools.ComponentItem):
    """type class - the name of a particular class (or any four-character code) """
    want = 'type'

class empty_ae_name_(aetools.ComponentItem):
    """ - the undefined value """
    want = 'undf'

class Unicode_text(aetools.ComponentItem):
    """Unicode text - text in the Unicode format (cannot be viewed without conversion) """
    want = 'utxt'

Unicode_text = Unicode_text

class vector(aetools.ComponentItem):
    """vector - An ordered collection of items """
    want = 'vect'

vectors = vector

class version(aetools.ComponentItem):
    """version - a version value """
    want = 'vers'

class Wednesday(aetools.ComponentItem):
    """Wednesday - Wednesday """
    want = 'wed '

class weekdays(aetools.ComponentItem):
    """weekdays -  """
    want = 'wkdy'

weekday = weekdays

class yards(aetools.ComponentItem):
    """yards - a distance measurement in Imperial yards """
    want = 'yard'

class zones(aetools.ComponentItem):
    """zones -  """
    want = 'zone'

zone = zones
anything._superclassnames = []
anything._privpropdict = {
}
anything._privelemdict = {
}
pictures._superclassnames = []
pictures._privpropdict = {
}
pictures._privelemdict = {
}
styled_text._superclassnames = []
styled_text._privpropdict = {
}
styled_text._privelemdict = {
}
styled_text._superclassnames = []
styled_text._privpropdict = {
}
styled_text._privelemdict = {
}
strings._superclassnames = []
strings._privpropdict = {
}
strings._privelemdict = {
}
alias._superclassnames = []
alias._privpropdict = {
    'POSIX_path' : _Prop_POSIX_path,
}
alias._privelemdict = {
}
April._superclassnames = []
April._privpropdict = {
}
April._privelemdict = {
}
August._superclassnames = []
August._privpropdict = {
}
August._privelemdict = {
}
booleans._superclassnames = []
booleans._privpropdict = {
}
booleans._privelemdict = {
}
RGB_colors._superclassnames = []
RGB_colors._privpropdict = {
}
RGB_colors._privelemdict = {
}
application._superclassnames = []
application._privpropdict = {
    'AppleScript' : _Prop_AppleScript,
    'days' : _Prop_days,
    'hours' : _Prop_hours,
    'minutes' : _Prop_minutes,
    'pi' : _Prop_pi,
    'print_depth' : _Prop_print_depth,
    'print_length' : _Prop_print_length,
    'result' : _Prop_result,
    'return_' : _Prop_return_,
    'space' : _Prop_space,
    'tab' : _Prop_tab,
    'text_item_delimiters' : _Prop_text_item_delimiters,
    'weeks' : _Prop_weeks,
}
application._privelemdict = {
}
upper_case._superclassnames = []
upper_case._privpropdict = {
}
upper_case._privelemdict = {
}
cubic_centimeters._superclassnames = []
cubic_centimeters._privpropdict = {
}
cubic_centimeters._privelemdict = {
}
cubic_feet._superclassnames = []
cubic_feet._privpropdict = {
}
cubic_feet._privelemdict = {
}
characters._superclassnames = []
characters._privpropdict = {
}
characters._privelemdict = {
}
writing_code_info._superclassnames = []
writing_code_info._privpropdict = {
    'language_code' : _Prop_language_code,
    'script_code' : _Prop_script_code,
}
writing_code_info._privelemdict = {
}
text_items._superclassnames = []
text_items._privpropdict = {
}
text_items._privelemdict = {
}
cubic_meters._superclassnames = []
cubic_meters._privpropdict = {
}
cubic_meters._privelemdict = {
}
centimeters._superclassnames = []
centimeters._privpropdict = {
}
centimeters._privelemdict = {
}
item._superclassnames = []
item._privpropdict = {
    'id' : _Prop_id,
}
item._privelemdict = {
}
C_strings._superclassnames = []
C_strings._privpropdict = {
}
C_strings._privelemdict = {
}
text._superclassnames = []
text._privpropdict = {
}
text._privelemdict = {
}
cubic_inches._superclassnames = []
cubic_inches._privpropdict = {
}
cubic_inches._privelemdict = {
}
cubic_yards._superclassnames = []
cubic_yards._privpropdict = {
}
cubic_yards._privelemdict = {
}
December._superclassnames = []
December._privpropdict = {
}
December._privelemdict = {
}
degrees_Celsius._superclassnames = []
degrees_Celsius._privpropdict = {
}
degrees_Celsius._privelemdict = {
}
degrees_Fahrenheit._superclassnames = []
degrees_Fahrenheit._privpropdict = {
}
degrees_Fahrenheit._privelemdict = {
}
degrees_Kelvin._superclassnames = []
degrees_Kelvin._privpropdict = {
}
degrees_Kelvin._privelemdict = {
}
reals._superclassnames = []
reals._privpropdict = {
}
reals._privelemdict = {
}
encoded_strings._superclassnames = []
encoded_strings._privpropdict = {
}
encoded_strings._privelemdict = {
}
constants._superclassnames = []
constants._privpropdict = {
}
constants._privelemdict = {
}
events._superclassnames = []
events._privpropdict = {
}
events._privelemdict = {
}
February._superclassnames = []
February._privpropdict = {
}
February._privelemdict = {
}
feet._superclassnames = []
feet._privpropdict = {
}
feet._privelemdict = {
}
Friday._superclassnames = []
Friday._privpropdict = {
}
Friday._privelemdict = {
}
file_specification._superclassnames = []
file_specification._privpropdict = {
    'POSIX_path' : _Prop_POSIX_path,
}
file_specification._privelemdict = {
}
gallons._superclassnames = []
gallons._privpropdict = {
}
gallons._privelemdict = {
}
grams._superclassnames = []
grams._privpropdict = {
}
grams._privelemdict = {
}
handlers._superclassnames = []
handlers._privpropdict = {
}
handlers._privelemdict = {
}
inches._superclassnames = []
inches._privpropdict = {
}
inches._privelemdict = {
}
international_text._superclassnames = []
international_text._privpropdict = {
}
international_text._privelemdict = {
}
international_text._superclassnames = []
international_text._privpropdict = {
}
international_text._privelemdict = {
}
January._superclassnames = []
January._privpropdict = {
}
January._privelemdict = {
}
July._superclassnames = []
July._privpropdict = {
}
July._privelemdict = {
}
June._superclassnames = []
June._privpropdict = {
}
June._privelemdict = {
}
reference_forms._superclassnames = []
reference_forms._privpropdict = {
}
reference_forms._privelemdict = {
}
kilograms._superclassnames = []
kilograms._privpropdict = {
}
kilograms._privelemdict = {
}
kilometers._superclassnames = []
kilometers._privpropdict = {
}
kilometers._privelemdict = {
}
keystroke._superclassnames = []
keystroke._privpropdict = {
    'key' : _Prop_key,
    'key_kind' : _Prop_key_kind,
    'modifiers' : _Prop_modifiers,
}
keystroke._privelemdict = {
}
pounds._superclassnames = []
pounds._privpropdict = {
}
pounds._privelemdict = {
}
date._superclassnames = []
date._privpropdict = {
    'date_string' : _Prop_date_string,
    'day' : _Prop_day,
    'month' : _Prop_month,
    'time' : _Prop_time,
    'time_string' : _Prop_time_string,
    'weekday' : _Prop_weekday,
    'year' : _Prop_year,
}
date._privelemdict = {
}
list._superclassnames = []
list._privpropdict = {
    'length' : _Prop_length,
    'rest' : _Prop_rest,
    'reverse' : _Prop_reverse,
}
list._privelemdict = {
}
liters._superclassnames = []
liters._privpropdict = {
}
liters._privelemdict = {
}
linked_list._superclassnames = []
linked_list._privpropdict = {
    'length' : _Prop_length,
}
linked_list._privelemdict = {
}
integers._superclassnames = []
integers._privpropdict = {
}
integers._privelemdict = {
}
list_or_record._superclassnames = []
list_or_record._privpropdict = {
}
list_or_record._privelemdict = {
}
list_2c__record_or_text._superclassnames = []
list_2c__record_or_text._privpropdict = {
}
list_2c__record_or_text._privelemdict = {
}
list_or_string._superclassnames = []
list_or_string._privpropdict = {
}
list_or_string._privelemdict = {
}
machines._superclassnames = []
machines._privpropdict = {
}
machines._privelemdict = {
}
March._superclassnames = []
March._privpropdict = {
}
March._privelemdict = {
}
May._superclassnames = []
May._privpropdict = {
}
May._privelemdict = {
}
meters._superclassnames = []
meters._privpropdict = {
}
meters._privelemdict = {
}
miles._superclassnames = []
miles._privpropdict = {
}
miles._privelemdict = {
}
months._superclassnames = []
months._privpropdict = {
}
months._privelemdict = {
}
Monday._superclassnames = []
Monday._privpropdict = {
}
Monday._privelemdict = {
}
missing_values._superclassnames = []
missing_values._privpropdict = {
}
missing_values._privelemdict = {
}
number_or_date._superclassnames = []
number_or_date._privpropdict = {
}
number_or_date._privelemdict = {
}
number_2c__date_or_text._superclassnames = []
number_2c__date_or_text._privpropdict = {
}
number_2c__date_or_text._privelemdict = {
}
numbers._superclassnames = []
numbers._privpropdict = {
}
numbers._privelemdict = {
}
November._superclassnames = []
November._privpropdict = {
}
November._privelemdict = {
}
number_or_string._superclassnames = []
number_or_string._privpropdict = {
}
number_or_string._privelemdict = {
}
references._superclassnames = []
references._privpropdict = {
}
references._privelemdict = {
}
October._superclassnames = []
October._privpropdict = {
}
October._privelemdict = {
}
ounces._superclassnames = []
ounces._privpropdict = {
}
ounces._privelemdict = {
}
class_._superclassnames = ['type_class']
class_._privpropdict = {
    '_3c_Inheritance_3e_' : _Prop__3c_Inheritance_3e_,
}
class_._privelemdict = {
}
prepositions._superclassnames = []
prepositions._privpropdict = {
}
prepositions._privelemdict = {
}
properties._superclassnames = []
properties._privpropdict = {
}
properties._privelemdict = {
}
writing_code._superclassnames = []
writing_code._privpropdict = {
}
writing_code._privelemdict = {
}
Pascal_strings._superclassnames = []
Pascal_strings._privpropdict = {
}
Pascal_strings._privelemdict = {
}
quarts._superclassnames = []
quarts._privpropdict = {
}
quarts._privelemdict = {
}
data._superclassnames = []
data._privpropdict = {
}
data._privelemdict = {
}
records._superclassnames = []
records._privpropdict = {
}
records._privelemdict = {
}
Saturday._superclassnames = []
Saturday._privpropdict = {
}
Saturday._privelemdict = {
}
seconds._superclassnames = []
seconds._privpropdict = {
}
seconds._privelemdict = {
}
script._superclassnames = []
script._privpropdict = {
    'name' : _Prop_name,
    'parent' : _Prop_parent,
}
script._privelemdict = {
}
September._superclassnames = []
September._privpropdict = {
}
September._privelemdict = {
}
alias_or_string._superclassnames = []
alias_or_string._privpropdict = {
}
alias_or_string._privelemdict = {
}
sounds._superclassnames = []
sounds._privpropdict = {
}
sounds._privelemdict = {
}
square_feet._superclassnames = []
square_feet._privpropdict = {
}
square_feet._privelemdict = {
}
square_kilometers._superclassnames = []
square_kilometers._privpropdict = {
}
square_kilometers._privelemdict = {
}
square_miles._superclassnames = []
square_miles._privpropdict = {
}
square_miles._privelemdict = {
}
square_meters._superclassnames = []
square_meters._privpropdict = {
}
square_meters._privelemdict = {
}
square_yards._superclassnames = []
square_yards._privpropdict = {
}
square_yards._privelemdict = {
}
styled_Clipboard_text._superclassnames = []
styled_Clipboard_text._privpropdict = {
}
styled_Clipboard_text._privelemdict = {
}
styled_Clipboard_text._superclassnames = []
styled_Clipboard_text._privpropdict = {
}
styled_Clipboard_text._privelemdict = {
}
Sunday._superclassnames = []
Sunday._privpropdict = {
}
Sunday._privelemdict = {
}
styled_Unicode_text._superclassnames = []
styled_Unicode_text._privpropdict = {
}
styled_Unicode_text._privelemdict = {
}
styled_Unicode_text._superclassnames = []
styled_Unicode_text._privpropdict = {
}
styled_Unicode_text._privelemdict = {
}
Thursday._superclassnames = []
Thursday._privpropdict = {
}
Thursday._privelemdict = {
}
Tuesday._superclassnames = []
Tuesday._privpropdict = {
}
Tuesday._privelemdict = {
}
type_class._superclassnames = []
type_class._privpropdict = {
}
type_class._privelemdict = {
}
empty_ae_name_._superclassnames = []
empty_ae_name_._privpropdict = {
}
empty_ae_name_._privelemdict = {
}
Unicode_text._superclassnames = []
Unicode_text._privpropdict = {
}
Unicode_text._privelemdict = {
}
Unicode_text._superclassnames = []
Unicode_text._privpropdict = {
}
Unicode_text._privelemdict = {
}
vector._superclassnames = []
vector._privpropdict = {
    'length' : _Prop_length,
}
vector._privelemdict = {
}
version._superclassnames = []
version._privpropdict = {
}
version._privelemdict = {
}
Wednesday._superclassnames = []
Wednesday._privpropdict = {
}
Wednesday._privelemdict = {
}
weekdays._superclassnames = []
weekdays._privpropdict = {
}
weekdays._privelemdict = {
}
yards._superclassnames = []
yards._privpropdict = {
}
yards._privelemdict = {
}
zones._superclassnames = []
zones._privpropdict = {
}
zones._privelemdict = {
}
_Enum_boov = {
    'true' : 'true',    # the true boolean value
    'false' : 'fals',   # the false boolean value
}

_Enum_cons = {
    'case' : 'case',    # case
    'diacriticals' : 'diac',    # diacriticals
    'white_space' : 'whit',     # white space
    'hyphens' : 'hyph', # hyphens
    'expansion' : 'expa',       # expansion
    'punctuation' : 'punc',     # punctuation
    'application_responses' : 'rmte',   # remote event replies
}

_Enum_eMds = {
    'option_down' : 'Kopt',     #
    'command_down' : 'Kcmd',    #
    'control_down' : 'Kctl',    #
    'shift_down' : 'Ksft',      #
    'caps_lock_down' : 'Kclk',  #
}

_Enum_ekst = {
    'escape_key' : 'ks5\x00',   #
    'delete_key' : 'ks3\x00',   #
    'tab_key' : 'ks0\x00',      #
    'return_key' : 'ks$\x00',   #
    'clear_key' : 'ksG\x00',    #
    'enter_key' : 'ksL\x00',    #
    'up_arrow_key' : 'ks~\x00', #
    'down_arrow_key' : 'ks}\x00',       #
    'left_arrow_key' : 'ks{\x00',       #
    'right_arrow_key' : 'ks|\x00',      #
    'help_key' : 'ksr\x00',     #
    'home_key' : 'kss\x00',     #
    'page_up_key' : 'kst\x00',  #
    'page_down_key' : 'ksy\x00',        #
    'forward_del_key' : 'ksu\x00',      #
    'end_key' : 'ksw\x00',      #
    'F1_key' : 'ksz\x00',       #
    'F2_key' : 'ksx\x00',       #
    'F3_key' : 'ksc\x00',       #
    'F4_key' : 'ksv\x00',       #
    'F5_key' : 'ks`\x00',       #
    'F6_key' : 'ksa\x00',       #
    'F7_key' : 'ksb\x00',       #
    'F8_key' : 'ksd\x00',       #
    'F9_key' : 'kse\x00',       #
    'F10_key' : 'ksm\x00',      #
    'F11_key' : 'ksg\x00',      #
    'F12_key' : 'kso\x00',      #
    'F13_key' : 'ksi\x00',      #
    'F14_key' : 'ksk\x00',      #
    'F15_key' : 'ksq\x00',      #
}

_Enum_misc = {
    'current_application' : 'cura',     # the current application
}


#
# Indices of types declared in this module
#
_classdeclarations = {
    '****' : anything,
    'PICT' : pictures,
    'STXT' : styled_text,
    'TEXT' : strings,
    'alis' : alias,
    'apr ' : April,
    'aug ' : August,
    'bool' : booleans,
    'cRGB' : RGB_colors,
    'capp' : application,
    'case' : upper_case,
    'ccmt' : cubic_centimeters,
    'cfet' : cubic_feet,
    'cha ' : characters,
    'citl' : writing_code_info,
    'citm' : text_items,
    'cmet' : cubic_meters,
    'cmtr' : centimeters,
    'cobj' : item,
    'cstr' : C_strings,
    'ctxt' : text,
    'cuin' : cubic_inches,
    'cyrd' : cubic_yards,
    'dec ' : December,
    'degc' : degrees_Celsius,
    'degf' : degrees_Fahrenheit,
    'degk' : degrees_Kelvin,
    'doub' : reals,
    'encs' : encoded_strings,
    'enum' : constants,
    'evnt' : events,
    'feb ' : February,
    'feet' : feet,
    'fri ' : Friday,
    'fss ' : file_specification,
    'galn' : gallons,
    'gram' : grams,
    'hand' : handlers,
    'inch' : inches,
    'itxt' : international_text,
    'jan ' : January,
    'jul ' : July,
    'jun ' : June,
    'kfrm' : reference_forms,
    'kgrm' : kilograms,
    'kmtr' : kilometers,
    'kprs' : keystroke,
    'lbs ' : pounds,
    'ldt ' : date,
    'list' : list,
    'litr' : liters,
    'llst' : linked_list,
    'long' : integers,
    'lr  ' : list_or_record,
    'lrs ' : list_2c__record_or_text,
    'ls  ' : list_or_string,
    'mach' : machines,
    'mar ' : March,
    'may ' : May,
    'metr' : meters,
    'mile' : miles,
    'mnth' : months,
    'mon ' : Monday,
    'msng' : missing_values,
    'nd  ' : number_or_date,
    'nds ' : number_2c__date_or_text,
    'nmbr' : numbers,
    'nov ' : November,
    'ns  ' : number_or_string,
    'obj ' : references,
    'oct ' : October,
    'ozs ' : ounces,
    'pcls' : class_,
    'prep' : prepositions,
    'prop' : properties,
    'psct' : writing_code,
    'pstr' : Pascal_strings,
    'qrts' : quarts,
    'rdat' : data,
    'reco' : records,
    'sat ' : Saturday,
    'scnd' : seconds,
    'scpt' : script,
    'sep ' : September,
    'sf  ' : alias_or_string,
    'snd ' : sounds,
    'sqft' : square_feet,
    'sqkm' : square_kilometers,
    'sqmi' : square_miles,
    'sqrm' : square_meters,
    'sqyd' : square_yards,
    'styl' : styled_Clipboard_text,
    'sun ' : Sunday,
    'sutx' : styled_Unicode_text,
    'thu ' : Thursday,
    'tue ' : Tuesday,
    'type' : type_class,
    'undf' : empty_ae_name_,
    'utxt' : Unicode_text,
    'vect' : vector,
    'vers' : version,
    'wed ' : Wednesday,
    'wkdy' : weekdays,
    'yard' : yards,
    'zone' : zones,
}

_propdeclarations = {
    'ID  ' : _Prop_id,
    'ascr' : _Prop_AppleScript,
    'c@#^' : _Prop__3c_Inheritance_3e_,
    'day ' : _Prop_day,
    'days' : _Prop_days,
    'dstr' : _Prop_date_string,
    'hour' : _Prop_hours,
    'kMod' : _Prop_modifiers,
    'kMsg' : _Prop_key,
    'kknd' : _Prop_key_kind,
    'leng' : _Prop_length,
    'min ' : _Prop_minutes,
    'mnth' : _Prop_month,
    'pare' : _Prop_parent,
    'pi  ' : _Prop_pi,
    'plcd' : _Prop_language_code,
    'pnam' : _Prop_name,
    'prdp' : _Prop_print_depth,
    'prln' : _Prop_print_length,
    'pscd' : _Prop_script_code,
    'psxp' : _Prop_POSIX_path,
    'rest' : _Prop_rest,
    'ret ' : _Prop_return_,
    'rslt' : _Prop_result,
    'rvse' : _Prop_reverse,
    'spac' : _Prop_space,
    'tab ' : _Prop_tab,
    'time' : _Prop_time,
    'tstr' : _Prop_time_string,
    'txdl' : _Prop_text_item_delimiters,
    'week' : _Prop_weeks,
    'wkdy' : _Prop_weekday,
    'year' : _Prop_year,
}

_compdeclarations = {
}

_enumdeclarations = {
    'boov' : _Enum_boov,
    'cons' : _Enum_cons,
    'eMds' : _Enum_eMds,
    'ekst' : _Enum_ekst,
    'misc' : _Enum_misc,
}



# Copyright 2006 Google, Inc. All Rights Reserved.
# Licensed to PSF under a Contributor Agreement.

"""Fixer that turns <> into !=."""

# Local imports
from .. import pytree
from ..pgen2 import token
from .. import fixer_base


class FixNe(fixer_base.BaseFix):
    # This is so simple that we don't need the pattern compiler.

    _accept_type = token.NOTEQUAL

    def match(self, node):
        # Override
        return node.value == u"<>"

    def transform(self, node, results):
        new = pytree.Leaf(token.NOTEQUAL, u"!=", prefix=node.prefix)
        return new



# Augmented assignment test.

from test.test_support import run_unittest, check_py3k_warnings
import unittest


class AugAssignTest(unittest.TestCase):
    def testBasic(self):
        x = 2
        x += 1
        x *= 2
        x **= 2
        x -= 8
        x //= 5
        x %= 3
        x &= 2
        x |= 5
        x ^= 1
        x /= 2
        if 1/2 == 0:
            # classic division
            self.assertEqual(x, 3)
        else:
            # new-style division (with -Qnew)
            self.assertEqual(x, 3.0)

    def test_with_unpacking(self):
        self.assertRaises(SyntaxError, compile, "x, b += 3", "<test>", "exec")

    def testInList(self):
        x = [2]
        x[0] += 1
        x[0] *= 2
        x[0] **= 2
        x[0] -= 8
        x[0] //= 5
        x[0] %= 3
        x[0] &= 2
        x[0] |= 5
        x[0] ^= 1
        x[0] /= 2
        if 1/2 == 0:
            self.assertEqual(x[0], 3)
        else:
            self.assertEqual(x[0], 3.0)

    def testInDict(self):
        x = {0: 2}
        x[0] += 1
        x[0] *= 2
        x[0] **= 2
        x[0] -= 8
        x[0] //= 5
        x[0] %= 3
        x[0] &= 2
        x[0] |= 5
        x[0] ^= 1
        x[0] /= 2
        if 1/2 == 0:
            self.assertEqual(x[0], 3)
        else:
            self.assertEqual(x[0], 3.0)

    def testSequences(self):
        x = [1,2]
        x += [3,4]
        x *= 2

        self.assertEqual(x, [1, 2, 3, 4, 1, 2, 3, 4])

        x = [1, 2, 3]
        y = x
        x[1:2] *= 2
        y[1:2] += [1]

        self.assertEqual(x, [1, 2, 1, 2, 3])
        self.assertTrue(x is y)

    def testCustomMethods1(self):

        class aug_test:
            def __init__(self, value):
                self.val = value
            def __radd__(self, val):
                return self.val + val
            def __add__(self, val):
                return aug_test(self.val + val)

        class aug_test2(aug_test):
            def __iadd__(self, val):
                self.val = self.val + val
                return self

        class aug_test3(aug_test):
            def __iadd__(self, val):
                return aug_test3(self.val + val)

        x = aug_test(1)
        y = x
        x += 10

        self.assertIsInstance(x, aug_test)
        self.assertTrue(y is not x)
        self.assertEqual(x.val, 11)

        x = aug_test2(2)
        y = x
        x += 10

        self.assertTrue(y is x)
        self.assertEqual(x.val, 12)

        x = aug_test3(3)
        y = x
        x += 10

        self.assertIsInstance(x, aug_test3)
        self.assertTrue(y is not x)
        self.assertEqual(x.val, 13)


    def testCustomMethods2(test_self):
        output = []

        class testall:
            def __add__(self, val):
                output.append("__add__ called")
            def __radd__(self, val):
                output.append("__radd__ called")
            def __iadd__(self, val):
                output.append("__iadd__ called")
                return self

            def __sub__(self, val):
                output.append("__sub__ called")
            def __rsub__(self, val):
                output.append("__rsub__ called")
            def __isub__(self, val):
                output.append("__isub__ called")
                return self

            def __mul__(self, val):
                output.append("__mul__ called")
            def __rmul__(self, val):
                output.append("__rmul__ called")
            def __imul__(self, val):
                output.append("__imul__ called")
                return self

            def __div__(self, val):
                output.append("__div__ called")
            def __rdiv__(self, val):
                output.append("__rdiv__ called")
            def __idiv__(self, val):
                output.append("__idiv__ called")
                return self

            def __floordiv__(self, val):
                output.append("__floordiv__ called")
                return self
            def __ifloordiv__(self, val):
                output.append("__ifloordiv__ called")
                return self
            def __rfloordiv__(self, val):
                output.append("__rfloordiv__ called")
                return self

            def __truediv__(self, val):
                output.append("__truediv__ called")
                return self
            def __itruediv__(self, val):
                output.append("__itruediv__ called")
                return self

            def __mod__(self, val):
                output.append("__mod__ called")
            def __rmod__(self, val):
                output.append("__rmod__ called")
            def __imod__(self, val):
                output.append("__imod__ called")
                return self

            def __pow__(self, val):
                output.append("__pow__ called")
            def __rpow__(self, val):
                output.append("__rpow__ called")
            def __ipow__(self, val):
                output.append("__ipow__ called")
                return self

            def __or__(self, val):
                output.append("__or__ called")
            def __ror__(self, val):
                output.append("__ror__ called")
            def __ior__(self, val):
                output.append("__ior__ called")
                return self

            def __and__(self, val):
                output.append("__and__ called")
            def __rand__(self, val):
                output.append("__rand__ called")
            def __iand__(self, val):
                output.append("__iand__ called")
                return self

            def __xor__(self, val):
                output.append("__xor__ called")
            def __rxor__(self, val):
                output.append("__rxor__ called")
            def __ixor__(self, val):
                output.append("__ixor__ called")
                return self

            def __rshift__(self, val):
                output.append("__rshift__ called")
            def __rrshift__(self, val):
                output.append("__rrshift__ called")
            def __irshift__(self, val):
                output.append("__irshift__ called")
                return self

            def __lshift__(self, val):
                output.append("__lshift__ called")
            def __rlshift__(self, val):
                output.append("__rlshift__ called")
            def __ilshift__(self, val):
                output.append("__ilshift__ called")
                return self

        x = testall()
        x + 1
        1 + x
        x += 1

        x - 1
        1 - x
        x -= 1

        x * 1
        1 * x
        x *= 1

        if 1/2 == 0:
            x / 1
            1 / x
            x /= 1
        else:
            # True division is in effect, so "/" doesn't map to __div__ etc;
            # but the canned expected-output file requires that those get called.
            x.__div__(1)
            x.__rdiv__(1)
            x.__idiv__(1)

        x // 1
        1 // x
        x //= 1

        x % 1
        1 % x
        x %= 1

        x ** 1
        1 ** x
        x **= 1

        x | 1
        1 | x
        x |= 1

        x & 1
        1 & x
        x &= 1

        x ^ 1
        1 ^ x
        x ^= 1

        x >> 1
        1 >> x
        x >>= 1

        x << 1
        1 << x
        x <<= 1

        test_self.assertEqual(output, '''\
__add__ called
__radd__ called
__iadd__ called
__sub__ called
__rsub__ called
__isub__ called
__mul__ called
__rmul__ called
__imul__ called
__div__ called
__rdiv__ called
__idiv__ called
__floordiv__ called
__rfloordiv__ called
__ifloordiv__ called
__mod__ called
__rmod__ called
__imod__ called
__pow__ called
__rpow__ called
__ipow__ called
__or__ called
__ror__ called
__ior__ called
__and__ called
__rand__ called
__iand__ called
__xor__ called
__rxor__ called
__ixor__ called
__rshift__ called
__rrshift__ called
__irshift__ called
__lshift__ called
__rlshift__ called
__ilshift__ called
'''.splitlines())

def test_main():
    with check_py3k_warnings(("classic int division", DeprecationWarning)):
        run_unittest(AugAssignTest)

if __name__ == '__main__':
    test_main()



"""Suite Enumerations: Enumerations for the Finder
Level 1, version 1

Generated from /System/Library/CoreServices/Finder.app
AETE/AEUT resource version 0/144, language 0, script 0
"""

import aetools
import MacOS

_code = 'tpnm'

from StdSuites.Type_Names_Suite import *
class Enumerations_Events(Type_Names_Suite_Events):

    pass

_Enum_earr = {
    'not_arranged' : 'narr',    #
    'snap_to_grid' : 'grda',    #
    'arranged_by_name' : 'nama',        #
    'arranged_by_modification_date' : 'mdta',   #
    'arranged_by_creation_date' : 'cdta',       #
    'arranged_by_size' : 'siza',        #
    'arranged_by_kind' : 'kina',        #
    'arranged_by_label' : 'laba',       #
}

_Enum_ecvw = {
    'icon_view' : 'icnv',       #
    'list_view' : 'lsvw',       #
    'column_view' : 'clvw',     #
}

_Enum_edfm = {
    'Mac_OS_format' : 'dfhf',   #
    'Mac_OS_Extended_format' : 'dfh+',  #
    'UFS_format' : 'dfuf',      #
    'NFS_format' : 'dfnf',      #
    'audio_format' : 'dfau',    #
    'ProDOS_format' : 'dfpr',   #
    'MS_2d_DOS_format' : 'dfms',        #
    'ISO_9660_format' : 'df96', #
    'High_Sierra_format' : 'dfhs',      #
    'QuickTake_format' : 'dfqt',        #
    'Apple_Photo_format' : 'dfph',      #
    'AppleShare_format' : 'dfas',       #
    'UDF_format' : 'dfud',      #
    'WebDAV_format' : 'dfwd',   #
    'FTP_format' : 'dfft',      #
    'Packet_2d_written_UDF_format' : 'dfpu',    #
    'unknown_format' : 'df??',  #
}

_Enum_elsv = {
    'name_column' : 'elsn',     #
    'modification_date_column' : 'elsm',        #
    'creation_date_column' : 'elsc',    #
    'size_column' : 'elss',     #
    'kind_column' : 'elsk',     #
    'label_column' : 'elsl',    #
    'version_column' : 'elsv',  #
    'comment_column' : 'elsC',  #
}

_Enum_ipnl = {
    'General_Information_panel' : 'gpnl',       #
    'Sharing_panel' : 'spnl',   #
    'Memory_panel' : 'mpnl',    #
    'Preview_panel' : 'vpnl',   #
    'Application_panel' : 'apnl',       #
    'Languages_panel' : 'pklg', #
    'Plugins_panel' : 'pkpg',   #
    'Name__26__Extension_panel' : 'npnl',       #
    'Comments_panel' : 'cpnl',  #
    'Content_Index_panel' : 'cinl',     #
}

_Enum_isiz = {
    'mini' : 'miic',    #
    'small' : 'smic',   #
    'large' : 'lgic',   #
}

_Enum_lvic = {
    'small_icon' : 'smic',      #
    'large_icon' : 'lgic',      #
}

_Enum_priv = {
    'read_only' : 'read',       #
    'read_write' : 'rdwr',      #
    'write_only' : 'writ',      #
    'none' : 'none',    #
}

_Enum_sodr = {
    'normal' : 'snrm',  #
    'reversed' : 'srvs',        #
}

_Enum_vwby = {
    'conflicts' : 'cflc',       #
    'existing_items' : 'exsi',  #
    'small_icon' : 'smic',      #
    'icon' : 'iimg',    #
    'name' : 'pnam',    #
    'modification_date' : 'asmo',       #
    'size' : 'ptsz',    #
    'kind' : 'kind',    #
    'comment' : 'comt', #
    'label' : 'labi',   #
    'version' : 'vers', #
    'creation_date' : 'ascd',   #
    'small_button' : 'smbu',    #
    'large_button' : 'lgbu',    #
    'grid' : 'grid',    #
    'all' : 'kyal',     #
}


#
# Indices of types declared in this module
#
_classdeclarations = {
}

_propdeclarations = {
}

_compdeclarations = {
}

_enumdeclarations = {
    'earr' : _Enum_earr,
    'ecvw' : _Enum_ecvw,
    'edfm' : _Enum_edfm,
    'elsv' : _Enum_elsv,
    'ipnl' : _Enum_ipnl,
    'isiz' : _Enum_isiz,
    'lvic' : _Enum_lvic,
    'priv' : _Enum_priv,
    'sodr' : _Enum_sodr,
    'vwby' : _Enum_vwby,
}
